/**
 * @summary     cytoscape.vizmapper
 * @description Add vizmapper to cytoscape.js graph area.
 * @version     
 * @file         cytoscape.vizmapper.js
 * @author      Alexan Andrieux
 * @contact     <a href="mailto:alexan.andrieux@inria.fr"> Alexan Andrieux</a>
 * @category
 *
 * @copyright 
 *
 * This source file is free software, under AGLP License
 * 
 * This source file is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the license files for details.
 * 
 * For details please refer to: <a href="http://www.gnu.org/licenses/agpl-3.0.html"></a>
 */

/**
 * Based on Gradient-picker.js Matt Crinklaw-Vogt (tantaman)
 * 
 * The vizmapper allow to define style in function of properties.
 * Properties can be :
 * - length
 * - coverage
 * 
 * Each properties can be associated with a style elements:
 * - shapes and size
 * - color
 * - fill (color and shape)
 * - border (color)
 * - label  (not fully implemented yet)
 * 
 * For association between properties and style elements, the order is important so the first property defined
 * will be associated with the first style elements.
 * Example:
 * 	if the properties is define by : ["Length", "Coverage"]
 *  and the style elements by  ["shapes", "colors"]
 *  The properties Length will be associated with the style elements shapes.
 * 
 * The initial configuration of distribution for nodes and edges is defined in the config files with "default_nodes_map" and "default_edges_map".
 * Example:
 * 		"default_nodes_map":["#ff0000 0% 1 ellipse 20 ", "#ffa500 50% 1 ellipse 30 ", "#008000 100% 1 ellipse 40 "],
 *		"default_edges_map":["#ff0000 0% 1 solid 3 ", "#ffa500 50% 1 solid 3 ", "#008000 100% 1 solid 3 "] 
 * 
 * In the config files some behavior can be confired:
 * - display vizmapper
 * - normal or advanced vizmapper 
 * - use JQuery accordion
 * - use hold root by default
 * - display reset button
 * - display type of mapper (gradient or discret)
 * 
 * There are two mode of vizmapper (normal are advanced):
 * - The normal mode is for a single style element like color and shape.
 *   With this one, there only one preview color or shape. 
 *   and the bubble pickers/setters will be adapted to define only one style property (color or shape and size). 
 * - The advanced allow to define several style elements in same time (color, shape, size).
 *   So it's better to use with fill and border.
 * 	 With this one, two preview are displed, one for shape and one for colors
 *   and the bubble pickers/setters will be adapted to define all style properties in same time(color, shape and size).    
 * 
 * NB:
 * The graph class object (here cy) must be contained a _private attribute (cy._private) with :
 * - the maximum length of sequence nodes (maxNodesLenght) 
 * - the maximum coverage of nodes (maxNodesCoverage) (undefined if coverage is not present) 
 * - the maximum coverage of edges (maxEdgesCoverage) (undefined if coverage is not present)
 * 
 * @module vizMapper
 * 
 * @requires <a href='http://blog.jquery.com/2011/09/12/jquery-1-6-4-released/'>jQuery Core 1.6.4</a>
 * @dependency <script language="JavaScript" type="text/javascript" src="../biojs/dependencies/jquery/jquery-1.4.2.min.js"></script>
 * 
 * @requires <a href='http://cytoscape.github.io/cytoscape.js/'>cytoscape 2.0.4+</a>
 * 
 * @example
 *	//Initialize a new instance of vizmapper with config file
 *	$(selector).vizMap({
 * 					group : "nodes",
 *					property : ["Coverage"],
 *					attrCss : ["colors"],
 *					type : "gradient",
 *					tableOfNodes : '#dataGraph_nodes', //Data table coverage reference
 *				}, 
 * 				config //the config file
 * 	);
 * 
 * 	//Initialize a new instance of vizmapper without config file
 * 	$(selector).vizMap({
 * 					group : "nodes",
 *					property : ["Coverage"],
 *					controlPoints : ["#ff0000 0% 1 ellipse 20 ", "#ffa500 50% 1 ellipse 30 ", "#008000 100% 1 ellipse 40 "]
 *					attrCss : ["colors"],
 *					type : "gradient",
 *					tableOfNodes : '#dataGraph_nodes',
 *				}); 
 * 				
 */

//TODO: support with vertical vizmapper
(function( $ ) {
"use strict";
	if (!$.event.special.destroyed) {
		$.event.special.destroyed = {
		    remove: function(o) {
		    	if (o.handler) {
		    		o.handler();
		    	};
		    }
		};
	}

	function bind(fn, ctx) {
		if (typeof fn.bind === "function") {
			return fn.bind(ctx);
		} else {
			return function() {
				fn.apply(ctx, arguments);
			};
		}
	}
	
	/**
	 * VizMap is a master class that contained all the others. This class define an vizmapper instance.
	 * An instance can have several mappers in function of several properties ( ex: length and coverage).
	 * For add a new mapper in the Vizmapper object, just add the name of the properties in opts.properties. 
	 * This class create a class object ElementProperty for each elements of opts.property. 
	 * And ElementPropzmappererty is a container for the class object VizMapSelection.
	 * @class VizMap
	 * @param {object} $el the JQuery selector.
	 * @param {object} opts the object contained the initials values of attributes.
	 * @constructor
	 */
	
	function VizMap ($el, opts){
		
		$.extend(this.opts, opts);
		
		var self =this;
		self.$el = $el;
		self.$el.css("position", "relative");
		self.opts = opts;
		self.propContainer =[];
		
		var propContainer = self.propContainer;
		var len = self.opts.property.length;
		//Define a new element in propContainer list (list of object associated to one property)
		for (var i = 0; i < len; ++i) {
			var property= self.createElProperty(self.opts.property[i], i);
			//property.listener = this;
			propContainer.push(property);
		}
		
		//Define scrollable value.
		setTimeout( function(){
			self.isScrollable();
		},100);				
	};	
	
	VizMap.prototype = {
		
		opts :{
					/**    
					 * Define vizMapSelection group (nodes are edges).
					 * @attribute group
					 * @type String
					 * @default "nodes"
					 *    
					 */
					group: "nodes",
					/**    
					 * Define number and property used for each vizMapSelection.
					 * The number depends to the array length and the property is a string name that can be :
					 * - length  for length of sequence nodes
					 * - coverage for coverage of sequence nodes or edges 
					 * @attribute property
					 * @type array
					 * @default ["length"]
					 *    
					 */
					property: ["length"],
					/**    
					 * Define the number of control point 
					 * and for each control point define the color, the position, the opacity, the shape and the size of a control point.
					 * The number depends to the array length 
					 * and the properties is defined by a string like "Hexcolor positionIn% opacityValue ShapeName sizeValue".
					 * Opacity is defined between 0 and 1 and 
					 * ShapeName can be: 
					 * - for nodes: ellipse, triangle, square, roundrectangle, pentagon, heptagon, hexagon, octogon.
					 * - for edges lines: solid, dashed, dotted
					 * - for edges arrows: square, circle, diamond, tee, none
					 * @attribute controlPoints
					 * @type array
					 * @default ["#FFF 0% 1 ellipse 30", "#000 100% 1 ellipse 30"]
					 *    
					 */
					controlPoints:["#FFF 0% 1 ellipse 30", "#000 100% 1 ellipse 30"],
					/**    
					 * Display JQuery accordion are not.
					 * @attribute accordion
					 * @type boolean
					 * @default false
					 *    
					 */
					accordion:false,
					/**    
					 * Define the orientation of the preview (vertical or horizontal).
					 * vertical value is not implemented yet.
					 * @attribute orientation
					 * @type String
					 * @default "horizontal"
					 * @depreciate   
					 */
					orientation: "horizontal",
					/**    
					 * Active center on nodes.
					 * @attribute center
					 * @type boolean
					 * @default false
					 *    
					 */
					center: false,
					/**    
					 * Define the type of vizMapSelection.
					 * The type is a string name that can be :
					 * - color : just the elements color and opacity (one color preview)
					 * - shape : just the elements shape and size (one shape preview)
					 * - fill : the elements color, shape, size and opacity in same time (one color preview and one shape preview) 
					 * - border : just the elements color and opacity (one color preview)
					 * - label : only color implemented
					 * 
					 * @attribute property
					 * @type array
					 * @default ["length"]
					 *    
					 */
					attrCss: ["color"],
					/**    
					 * Define the color distribution.
					 * The color distribution can be gradient or discrete.
					 * @attribute type
					 * @type string
					 * @default gradient
					 *    
					 */
					type: "gradient",
					/**    
					 * Define the div id of nodes dataTable if is present.
					 * It's needed for update data table.
					 * @attribute tableOfNodes
					 * @type string
					 * @default undefined
					 *    
					 */
					tableOfNodes:undefined,
					/**    
					 * Define the div id of edges dataTable if is present.
					 * It's needed for update data table.
					 * @attribute tableOfEdges
					 * @type string
					 * @default undefined
					 *    
					 */
					tableOfEdges:undefined,
					/**    
					 * Define the div class if biojs.Sequence area if is present.
					 * It's needed for update header of sequence text.
					 * @attribute tableOfEdges
					 * @type string
					 * @default undefined
					 *    
					 */
					sequenceHeader : undefined,
					/**    
					 * Use advanced mode are not.
					 * @attribute advanced
					 * @type boolean
					 * @default undefined
					 *    
					 */
					advanced:false,
					/**    
					 * Lock the root node style.
					 * @attribute holdRoot
					 * @type boolean
					 * @default true
					 *    
					 */
					holdRoot : true,
					/**    
					 * Display radio button to choose gradient or discrete color distribution.
					 * @attribute holdRoot
					 * @type boolean
					 * @default false
					 *    
					 */
					typeChooser:false,
					/**    
					 * Display the rest button.
					 * @attribute resetButton
					 * @type boolean
					 * @default false
					 *    
					 */
					resetButton:true,
					/**    
					 * Display the checkbox for enable/disable center on nodes function.
					 * @attribute centerButton
					 * @type boolean
					 * @default false
					 *    
					 */
					centerButton : true
					//gradType: "linear",
					//change: function() {}
		}, 
		
		/**
		 * Create a new ElementProperty in propContainer.
		 * @method createElProperty
		 * @param {String} property the name of the property
		 * @param {int} index the position of this property in the list of properties.
		 * @private
		 */
		createElProperty:function(property, index){
			return new ElementProperty (this.$el, property, this.opts, index, this);
				
		},
		/**
		 * Update opts of VizMap instance
		 * @method updateOptions
		 * @param {object} opts new attributes values for update set this vizmapper instance.
		 * 
		 */
		updateOptions: function(opts) {
			var vizMap = $("#vizmapper").data("VizMap");
			var CurrentOpts = vizMap.opts;
			$.extend(CurrentOpts, opts);
			
			var nbElementProperty = vizMap.propContainer.length;
			for (var i=0; i<nbElementProperty; i++){
				vizMap.propContainer[i].attrCss.VizMapSelection.updateOptions(CurrentOpts);
			}
		},
		/**
		 * Define the scollable value. This values define if the vizMap have scroll bar displayed are not.
		 * @method isScrollable
		 * @private
		 */
		isScrollable: function() {
  			var divWidth = parseInt($("#side_content").height());
  			var scroll    = $("#vizmapper.content").height();
 			this.scrollable = (divWidth < scroll);
		},
		
		/**
		 * Adjust the size and position of control point after put ElementProperty/VizMapSelection in JQuery accordion.
		 * @method adjustToAccordion
		 * @param {int} outerWidth the outerWidth of VizMapSelection
		 * @param {int} timeout the timeout after it can be possible to ajust
		 * @depreciate
		 * @beta
		 * @private
		 */
		//TODO: need optimization because resize is long with 2 loops  
		adjustToAccordion : function(outerWidth, timeout){
		
			var originalCanvasWidth = $(".ui-continuous_map").width()-outerWidth;
			var originalCtrlPtPos=[];
			var i =0;
			
			//loop needed for support setTimeout write after
			$(".vizmapperPicker-ctrlPt").each(function(){
				    				var $this = $(this);	
				    				originalCtrlPtPos[i] = parseFloat($this.css("left"));
				    				i++;
			});
			
			this.$el.accordion();
			// setTimeout for catch the good width (time to make accordion and time for display scrollbar or remove it);
			setTimeout(function(){
	
						var newCanvasWidth = $(".ui-continuous_map").width()-outerWidth;
								
				    	$(".ui-continuous_map > canvas").each(function(){
				    				$(this).attr("width", newCanvasWidth);					
						});
						
						var scaleFactor = newCanvasWidth / originalCanvasWidth;
						var j =0;
						$(".vizmapperPicker-ctrlPt").each(function(){
				    				var $this = $(this);	
				    				var newCtrlPtPos =  parseInt(originalCtrlPtPos[j] * scaleFactor);
				    				$this.css("left", newCtrlPtPos);
				    				$this.data().posChanged(newCtrlPtPos);
				    				j++;	
				    				   					   		  				
				    	});
			    	
		   },timeout);

		},
		
		/**
		 * Add table of coverage files in vizmapper. This Table allow show the select coverage file and select an other one by click.
		 * This method is compatible with dataTable (nodes and edges) and Biojs.Sequence: 
		 * On change coverage file  data tables and header of biojs.Sequence area (the sequence text display by biojs.Sequence) 
		 * are updated with the new value of average coverage.  
		 * @method addFileCoverageTable
		 * @param {boolean} JQuery accordion is displayed are not.
		 * @private
		 */
		addFileCoverageTable : function(opts){
			var x = 0;
			while(cy.$("#n"+x) == undefined){
					x++;
			}
			var arrayCoverage= cy.$("#n"+x).data("coverage");
			
			if(arrayCoverage != undefined){
				this.$el.append(
							'<div id="coverageFiles" class="content">\
								<label class="files ui-widget-header ui-corner-tl ui-corner-tr ui-helper-clearfix">Coverage Files :</label>\
								<table cellpadding="0" cellspacing="0" border="0" class="display dataCoverage" id="table_coverageFiles">\
								    	  <thead>\
												<tr>\
												<th>File</th>\
												<th>Global avg coverage</th>\
												</tr>\
										 </thead>\
								</table>\
							</div>');
			
				
				var tableCoverageFiles = [];
				var nbFiles =  arrayCoverage.length;
				var nbNodes = cy.nodes().length;
				var nbEdges = cy.edges().length;
							
				//define global average coverage for each coverage files 			
				var globalAvgCoverage = [];
				var lenCoverage;
				var avg;
						
				for(var l=0; l<nbFiles; l++){
		    		lenCoverage=0;	
		    		avg =0;
			    	//for(var n=0; n<nbNodes; n++){
			    	cy.nodes().each(function(){
			    		if(this.data("coverage") == undefined){
			    			return true;
			    		} else if(this.data("coverage").length == 0){
			    			return true;
			    		} else if(this.data().coverage[l].avg_coverage == "none"){
			    			return true;
			    		}
			    					
			    		lenCoverage++;
			    		avg += this.data().coverage[l].avg_coverage;
			    		globalAvgCoverage[l] = avg;
			    	});
			    			
			    	//for(var e=0; e<nbEdges; e++){
			    	cy.edges().each(function(){	
			    		if(this.data("coverage") == undefined){
			    			return true;
			    		}else if(this.data("coverage").length == 0){
			    			return true;
			    		}else if(this.data().coverage[l].avg_coverage == "none"){
			    			return true;
			    		}
			    		
			    		lenCoverage++; 
			    		avg += this.data().coverage[l].avg_coverage;
			    		globalAvgCoverage[l] = avg;
			    	});
			    				
			    	globalAvgCoverage[l]= (globalAvgCoverage[l]/lenCoverage).toFixed(4);
				}
	
				for (var j=0; j<nbFiles; j++){
					tableCoverageFiles[j]={
						"Files":arrayCoverage[j].id,
						"Global avg coverage":globalAvgCoverage[j]
					};		
				};
							
				//// Initialize dataTable with coverage file data ////
				$('#table_coverageFiles').dataTable({       										
					"bPaginate": false,
					"bFilter": false,
					"bInfo": false,
					"bProcessing": true,
					"bJQueryUI": true,
					"oTableTools": {
						"sRowSelect": "single"
					},
					"aaData": tableCoverageFiles,
					"aoColumns": [
					      { "mData": "Files" },
					      { "mData": "Global avg coverage" }
					 ]
				});
					
				//remove header part
				$(".dataTables_wrapper").children(":first").remove();
					    
				//highlight the current coverage file
				var x = 0;
				while(cy.$("#n"+x) == undefined){
						x++;
				}
				var refNode = cy.$("#n"+x);	
				var defaultSelectedFile = refNode.data("coverageFileSelected");
	    		$("#table_coverageFiles td").each(function(){
	    			if($(this).text()==defaultSelectedFile){
	    				$(this).parent().addClass('selected');
	    			}
	    		});
		 
				//define click on coverage file action
				var fileSelected ={};
				$('#table_coverageFiles tr').click( function(){
					var $this = $(this);
							
					//highlight
					if ( !$this.hasClass('selected') ) {
						$this.parent().find(".selected").removeClass('selected');
						$this.addClass('selected');
					}
							
					fileSelected = $this.children(":first").text();
						   
					 //define the new value for each node
					var vizSelActif;
					if(opts.accordion){
						vizSelActif = $(".property_filters[aria-hidden=false]").children(".coverage-content");	
					}else{
						vizSelActif = $(".property_filters").children(".coverage-content");
					}
	
					//var opts = vizSelActif.data("VizMapSelection").opts;
					
					if(opts.tableOfNodes != undefined){	
						var indexNodes = 0;
						var maxCoverage = 0;
						cy.nodes().each(function(){
							var self = this;
							self.data("coverageFileSelected", fileSelected); 
							var coverage = self.data("coverage");
								   	
							//add support if coverage is undefined
							if(coverage == undefined){
								 coverage = [];
							}
								   	
							var lenCoverage = coverage.length;
							for(var k=0; k<lenCoverage; k++){
								if(coverage[k].id == fileSelected){
								 	//update coverage in nodes datatable 
								   	self.data("coverageValSelected", coverage[k].avg_coverage);
								   	if(coverage[k].avg_coverage != "none" && coverage[k].avg_coverage != undefined && maxCoverage< coverage[k].avg_coverage){
								   		maxCoverage = coverage[k].avg_coverage;
								   	};
								   			
								   	self.updateTable(opts.tableOfNodes, indexNodes, 2, coverage[k].id, maxCoverage);
								   	indexNodes++;
								   	break;
								  };
								   		
							 }
						});
						
					}
	
					if(opts.tableOfEdges != undefined){	
						var indexEdges = 0;
						var maxCoverage = 0;
						cy.edges().each(function(){
							var self = this;
							self.data("coverageFileSelected", fileSelected);
								   		
							var coverage = self.data("coverage");
							var lenCoverage = coverage.length;
							for(var l=0; l<lenCoverage; l++){
								if(coverage[l].id == fileSelected){
									this.data("coverageValSelected", coverage[l].avg_coverage);
								   	//update coverage in nodes datatable 
								   	if( coverage[l].avg_coverage != "none" && coverage[l].avg_coverage != undefined && maxCoverage< coverage[l].avg_coverage){
								   		maxCoverage = coverage[l].avg_coverage;
								   	};
								   	
								   	self.updateTable(opts.tableOfEdges, indexEdges, 4, coverage[l].id, maxCoverage);
								   	indexEdges++;
								   	break;
								   };
								}
							});
						}
						
						if($(opts.sequenceHeader).get(0) !=undefined){
							$(opts.sequenceHeader).data("biojsData")._redraw();
							
							var x = 0;
							while(cy.$("#n"+x) == undefined){
									x++;
							}
							var refNode = cy.$("#n"+x);
							
							$(".pane-south-data-value[file="+refNode.data("coverageFileSelected")+"]").removeClass("unselected").addClass("selected");
			    			$(".pane-south-data-value[file]").not("[file="+refNode.data("coverageFileSelected")+"]").removeClass("selected").addClass("unselected");
						}	
						//update scale of vizmapper and max value 
						var unactiveTab = $("#vizmapper_tabs > ul > li").not(".ui-state-active").children(); 
						unactiveTab.click();
						unactiveTab = $("#vizmapper_tabs > ul > li").not(".ui-state-active").children();
						unactiveTab.click();
				});		
			} 
		}	
	};
	
	
	/**
	 * ElementProperty define a class object and add html elements for each properties. 
	 * It's a container for VizMapSelection class.
	 * @class ElementProperty
	 * @param {object} $parentEl the JQuery selector.
	 * @param {String} property the name the property
	 * @param {object} opts the object contained the initials values of attributes.
	 * @param {int} index the position of the property in list of properties (propContainer). 
	 * @param {object} listener the listener sub-object of VizMap object 
	 * @constructor
	 */
	
	function ElementProperty($parentEl, property, opts, index, listener) {
		var toLowerCaseProperty = property.toLowerCase();
		var $el = $('<label class="property ui-widget-header ui-corner-tl ui-corner-tr ui-helper-clearfix" property=' + toLowerCaseProperty + '>' + property + ': </label><div class="property_filters" property =' + property + '></div>');
		var attrCss = [];
		var $childEl;
		var extendLabel;

		var self = this;
		self.$el = $el;
		self.opts = opts;
		self.attrCss = [];
		self.property = toLowerCaseProperty;
		self.listener = listener;

		var attributSet = opts.attrCss[index];
		var len = attributSet.length;

			if (opts.group == "nodes") {
				if (attributSet == "shapes") {
					extendLabel = " and sizes";
				} else {
					extendLabel = "";
				}
				$childEl = $('<div class="ui-continuous_map ui-discrete ' + toLowerCaseProperty + '-content ui-state-default" property="nodes_' + attributSet + '"><label class="style ">' + attributSet + extendLabel + ' : </label></div>');
			}else if (opts.group == "edges") {
				$childEl = $('<div class="ui-continuous_map ui-discrete ' + toLowerCaseProperty + '-content ui-state-default" property="edges_' + attributSet + '"><label class="style">' + attributSet + ' : </label></div>');
			};
			
			self.$el.filter('div').append($childEl);
			self.attrCss = {
				"type" : attributSet,
				"$el" : $childEl
			};


		//If JQuery accordion is display add event click on each label
		if(opts.accordion){
			$el.click(function() {
				var vizSelActif = self.attrCss;
				var vizSelActifLen = attrCss.length;
				attrCss.VizMapSelection.updatePreview();
				attrCss.VizMapSelection.updateView();
			});
		}

		$parentEl.append($el);
		self.$parentEl = $parentEl;
	};	

	/**
	 * VizMapSelection define a class object and add html elements for each ElementProperty:
	 * - add preview element (canvas), two preview is available one for colors and one for shapes. 
	 * - create a ControlPtConfig object that allow to configure or delete the control point.
	 * - add event to create controls points.
	 * - add event to update opts values, view and preview.
	 * - add initial controls points define in opts.
	 * 
	 * Html element of ControlPtConfig is hidden, it become visible on click on a control point.
	 * @class VizMapSelection
	 * @param {object} $el the JQuery selector.
	 * @param {String} property the name the property
	 * @param {object} opts the object contained the initials values of attributes.
	 * @param {object} listener the listener sub-object of VizMap object 
	 * @constructor
	 */
	function VizMapSelection( attrCss, opts, property, listener) {
			
		
		this.$el = attrCss.$el;
		this.$el.css("position", "relative");
		this.property = property;
		this.opts = opts;
		this.listener = listener;
		this.attrCss = attrCss.type;

		//Define preview with canvas
		var widthCorrection;
		if(this.opts.advanced){
			var $previewShape = $("<canvas class='vizmapperPicker-preview-shape'></canvas>");
			this.$el.append($previewShape);
			var canvasShape = $previewShape[0];	
			canvasShape.width = parseFloat(canvasShape.clientWidth);
			canvasShape.height = canvasShape.clientHeight;
			this.g2d = canvasShape.getContext("2d");
			
			var $previewColor = $("<canvas class='vizmapperPicker-preview-color'></canvas>");
			this.$el.append($previewColor);
			var canvasColor = $previewColor[0];
			canvasColor.width = parseFloat(canvasColor.clientWidth);
			canvasColor.height = canvasColor.clientHeight;
			this.g2d = canvasColor.getContext("2d");
		}else if( this.attrCss == "shapes"){
			var $previewShape = $("<canvas class='vizmapperPicker-preview-shape'></canvas>");
			this.$el.append($previewShape);
			var canvasShape = $previewShape[0];	
			canvasShape.width = parseFloat(canvasShape.clientWidth);
			canvasShape.height = canvasShape.clientHeight;
			this.g2d = canvasShape.getContext("2d");
		}else if(this.attrCss == "colors" ){
			var $previewColor = $("<canvas class='vizmapperPicker-preview-color'></canvas>");
			this.$el.append($previewColor);
			var canvasColor = $previewColor[0];
			canvasColor.width = parseFloat(canvasColor.clientWidth);
			canvasColor.height = canvasColor.clientHeight;
			this.g2d = canvasColor.getContext("2d");
		}


		/**
		 * Define a class object ControlPtConfig and add html elements for each instance of VizMapSelection.
		 * ControlPtConfig allow to do some action on a control point.
		 * Html element of ControlPtConfig is hidden, it become visible on click on a control point.
		 */ 
		var $ctrlPtContainer = $("<div class='vizmapperPicker-ctrlPts'></div>");
		this.$el.append($ctrlPtContainer);
		this.$ctrlPtContainer = $ctrlPtContainer;
		this.controlPoints = [];
		var len = opts.controlPoints.length;
		this.ctrlPtConfig = new ControlPtConfig(this.$el, opts, this.attrCss);
		for (var i = 0; i < len; ++i) {
			//Create the controls points define in opts (initials controls points)
			var ctrlPt = this.createCtrlPt(opts.controlPoints[i]);
			this.controlPoints.push(ctrlPt);
		}

		//Define events
		this.updatePreview = bind(this.updatePreview, this);
		this.updateView = bind(this.updateView, this);
		this.drawPreviewShape = bind(this.drawPreviewShape, this);
		this.doClicked = bind(this.doClicked, this);
		this.destroyed = bind(this.destroyed, this);
		$(document).bind("mouseup", this.doClicked);
		this.$el.bind("destroyed", this.destroyed);
		this.previewClicked = bind(this.previewClicked, this);
		
		if($previewColor != undefined){
			$previewColor.click(this.previewClicked);
		} 
		
		if($previewShape != undefined){
			$previewShape.click(this.previewClicked);
		}
		
		//update preview after created is done
		this.updatePreview();
	}

	VizMapSelection.prototype = {
		/**
		 * Hide html element of ctrlPtConfig.
		 * @event doClicked
		 */
		doClicked: function() {
			this.ctrlPtConfig.hide();
		},
		
		/**
		 * Create a control point.
		 * @event createCtrlPt
		 * @param {object} ctrlPtSetup the initials properties of the new control point.
		 */
		createCtrlPt: function(ctrlPtSetup) {
			return new ControlPoint(this.$ctrlPtContainer, ctrlPtSetup, this.opts.orientation, this, this.ctrlPtConfig);
	
		},

		/**
		 * Unbind click on ctrlPtConfig.
		 * @event destroyed
		 */
		destroyed: function() {
			$(document).unbind("click", this.doClicked);
		},

		/**
		 * Update opts values of VizMapSelection.
		 * @event updateOptions
		 * @param {object} opts opts object with new values of attributes
		 */
		updateOptions: function(opts) {
			var vizSelActif;
			if(opts.accordion){
				vizSelActif = $(".property_filters[aria-hidden=false]").children();
			}else{
				vizSelActif = $(".property_filters").children();
			}
			
			var CurrentOpts = vizSelActif.data("VizMapSelection").opts;
			$.extend(CurrentOpts, opts);
			
			this.updatePreview();
			this.updateView();
		},
		
		/**
		 * Update graph view.
		 * @event updateView
		 */
		updateView: function () {
	
			var arrayColor=[], 
				arraySeq=[], 
				arrayShape=[], 
				arrayOpacity=[], 
				arraySize=[], 
				/*arrayType=[],*/ 
				arrayCoverage =[];
	
			var len = this.controlPoints.length;
			for (var i = 0; i < len; i++) {
				var pt = this.controlPoints[i];
	
				arrayColor[i] = $.Color(pt.color).toHexString();
				arraySeq[i] = pt.lengthSeq;
				arrayShape[i] = pt.shape;
				arrayOpacity[i] = pt.opacity;
				arraySize[i] = pt.size;
				//arrayType[i] = pt.type;
				arrayCoverage[i] = pt.coverage;
	
				var propertyName = this.property;
				var holdRoot = this.opts.holdRoot;
	
				//Apply distribution of colors, shapes, sizes, opacity in function of controls points defined 
				if (propertyName == "length") {
					if(!this.opts.advanced) {
						arrayColor[i] = undefined;
						arrayOpacity[i] = undefined;
					}

					if(this.opts.type == "discrete") {
						if (arraySeq[i - 1] != undefined) {
							this.vizSwitcherDisc(this.$el.attr("property"), arraySeq[i - 1], arrayColor[i - 1], arrayShape[i - 1], arrayOpacity[i - 1], arraySize[i - 1], propertyName, holdRoot);
						} else if (arraySeq[i + 1] != undefined) {
							this.vizSwitcherDisc(this.$el.attr("property"), arraySeq[i], arrayColor[i], arrayShape[i], arrayOpacity[i], arraySize[i], propertyName, holdRoot);
						};
					}else if (this.opts.type == "gradient") {
						if (arraySeq[i - 1] != undefined) {
							this.vizSwitcherGrad(this.$el.attr("property"), arraySeq[i - 1], arraySeq[i], arrayColor[i - 1], arrayColor[i], arrayShape[i - 1], arrayOpacity[i - 1], arraySize[i - 1], arraySize[i], propertyName, holdRoot);
						}else if (arraySeq[i + 1] != undefined) {
							this.vizSwitcherGrad(this.$el.attr("property"), arraySeq[i], arraySeq[i + 1], arrayColor[i], arrayColor[i + 1], arrayShape[i], arrayOpacity[i], arraySize[i], arraySize[i + 1], propertyName, holdRoot);
						};
					};
				} else if(propertyName == "coverage") {
					if(!this.opts.advanced) {
						arrayShape[i] = undefined;
						arraySize[i] = undefined;
					}
					var coverageSelection = "coverageValSelected";
					if (this.opts.type == "discrete") {
						if (arrayCoverage[i - 1] != undefined) {
							this.vizSwitcherDisc(this.$el.attr("property"), arrayCoverage[i - 1], arrayColor[i - 1], arrayShape[i - 1], arrayOpacity[i - 1], arraySize[i - 1], coverageSelection);
						} else if (arrayCoverage[i + 1] != undefined) {
							this.vizSwitcherDisc(this.$el.attr("property"), arrayCoverage[i], arrayColor[i], arrayShape[i], arrayOpacity[i], arraySize[i], coverageSelection);
						};
					} else if(this.opts.type == "gradient") {
						if(arrayCoverage[i - 1] != undefined) {
							this.vizSwitcherGrad(this.$el.attr("property"), arrayCoverage[i - 1], arrayCoverage[i], arrayColor[i - 1], arrayColor[i], arrayShape[i - 1], arrayOpacity[i - 1], arraySize[i - 1], arraySize[i], coverageSelection);
						}else if (arrayCoverage[i + 1] != undefined) {
							this.vizSwitcherGrad(this.$el.attr("property"), arrayCoverage[i], arrayCoverage[i + 1], arrayColor[i], arrayColor[i + 1], arrayShape[i], arrayOpacity[i], arraySize[i], arraySize[i + 1], coverageSelection);
						};
					};
				}
	
			};
	
			this.arrayColor = arrayColor;
			this.arraySeq = arraySeq;
			this.arrayShape = arrayShape;
			this.arrayOpacity = arrayOpacity;
			this.arrayCoverage = arrayCoverage;
	
		},
	
		/**
		 * Update preview.
		 * @event updatePreview
		 */
		updatePreview: function() {
			var result = [];
			
			this.controlPoints.sort(this.ctrlPtComparator);
			var len = this.controlPoints.length;
	
			if (this.opts.type == "discrete") {
	
				for (var i = 0; i < len; i++) {
					var pt = this.controlPoints[i];
	
					result.push({
						position : pt.position,
						renderPos : pt.renderPos,
						color : pt.color,
						length : pt.length,
						shape : pt.shape,
						opacity : pt.opacity,
						size : pt.size,
						//type: pt.type,
						coverage : pt.coverage
					});
	
					if (this.opts.advanced || this.attrCss == "colors") {
						var fillRectWidth = this.g2d.canvas.width - pt.renderPos;
						this.g2d.fillStyle = pt.color;
						this.g2d.fillRect(pt.renderPos, 0, fillRectWidth, this.g2d.canvas.height);
					};
	
				};
			} else if (this.opts.type == "gradient") {
				if(this.attrCss == "colors"){
					var grad = this.g2d.createLinearGradient(0, 0, this.g2d.canvas.width, 0);
					for (var i = 0; i < len; i++) {
						var pt = this.controlPoints[i];				
						
						if(pt.position>1){
							pt.position = 1;
						}
						
						grad.addColorStop(pt.position, pt.color);
						result.push({
							position : pt.position,
							renderPos : pt.renderPos,
							color : pt.color,
							length : pt.length,
							shape : pt.shape,
							opacity : pt.opacity,
							size : pt.size,
							//type: pt.type,
							coverage : pt.coverage
						});
		
					};
		
					
					grad.addColorStop(pt.position, pt.color);
					this.g2d.fillStyle = grad;
					this.g2d.fillRect(0, 0, this.g2d.canvas.width, this.g2d.canvas.height);
				}else if(this.attrCss == "shapes"){
					
					for (var i = 0; i < len; i++) {
						var pt = this.controlPoints[i];				
						
						if(pt.position>1){
							pt.position = 1;
						}
						
						result.push({
							position : pt.position,
							renderPos : pt.renderPos,
							color : pt.color,
							length : pt.length,
							shape : pt.shape,
							opacity : pt.opacity,
							size : pt.size,
							//type: pt.type,
							coverage : pt.coverage
						});
		
					};
		
					if (this.opts.advanced){
						grad.addColorStop(pt.position, pt.color);
						this.g2d.fillStyle = grad;
						this.g2d.fillRect(0, 0, this.g2d.canvas.width, this.g2d.canvas.height);
					};
				}
				
	
			};
		},

		/**
		 * Delete a control point.
		 * @event updatePreview
		 * @param {object} ctrlPt the control point to delete.
		 */
		removeControlPoint: function(ctrlPt) {
			var cpidx = this.controlPoints.indexOf(ctrlPt);	
			if (cpidx != -1) {
			    ctrlPt.colorChanged("#000");
				this.controlPoints.splice(cpidx, 1);
				ctrlPt.$el.remove();
			}
		},
		
		/**
		 * Add a new control point on click on preview.
		 * @event previewClicked
		 * @param {object} e this event.
		 */
		previewClicked: function(e) {
			
			//get the position of mouse to add control point to the right position
			var offset = $(e.target).offset();
			var x = e.pageX - offset.left;
			var y = e.pageY - offset.top;
			
			var initialState = this.opts.controlPoints[0];
			var init = {};
			var $elAttr = this.$el.attr("property");
		
			//Initilize properties 
			if (typeof initialState === "string") {
				initialState = initialState.split(" ");
				init.position = parseFloat(initialState[1])/100;
				init.renderPos = x;
				init.color = initialState[0];
				init.opacity = initialState[2];
				init.shape = initialState[3];
				init.size = initialState[4];	
				//init.type = initialState[5];
			} else {
				init.position = initialState.position;
				init.renderPos = initialState.renderPos;
				init.color = initialState.color;
				init.opacity = initialState.opacity;
				init.shape = initialState.shape;
				init.size = initialState.size;
				//init.type = initialState.type;
				
			}
			
			var cp = this.createCtrlPt({
				position: x / this.g2d.canvas.width,
				renderPos :x,
				color: init.color,
				opacity: init.opacity,
				shape: init.shape,
				size: init.size,
				//type: init.type
			});
			//var imgData = this.g2d.getImageData(x,y,1,1);
			//var colorStr = "rgba(" + imgData.data[0] + "," + imgData.data[1] + "," + imgData.data[2] + "," + imgData.data[3] + ")";
			
			this.controlPoints.push(cp);
			this.controlPoints.sort(this.ctrlPtComparator);
			this.updatePreview();
		},
		
		/**
		 * Draw shape in canvas in function of the shape properties.
		 * @method drawPreviewShape
		 * @param {object} $previewShape the preview canvas.
		 * @param {object} ctrlPt the new control point.
		 * @private
		 */
		//TODO: Do the same without jcanvas
		//TODO: Add star shape
		drawPreviewShape: function ($previewShape, ctrlPt){
			var canvasShape={};
			canvasShape.$previewShape = $previewShape;
			
			var linePos = ctrlPt.position * $previewShape.width();
			var lineSize = $previewShape.height();
			var xShape = linePos+12;
			var yShape = lineSize/2;
			var colorShape;
			
			if(this.opts.advanced){
				colorShape = ctrlPt.color;
			}else{
				colorShape = "#000";
			}
			
			var strPosProperty;
			
			if(ctrlPt.property=="length"){
				strPosProperty = JSON.stringify(ctrlPt.lengthSeq);
			}else if(ctrlPt.property=="coverage"){
				strPosProperty = JSON.stringify(ctrlPt.coverage);
			}
			var ctrlPtnameLine = "ctrlPt"+strPosProperty+"Line";
			var ctrlPtnameShape = "ctrlPt"+strPosProperty+"Shape";
			var ctrlPtgroup = "ctrlPt"+strPosProperty+"Group";
			canvasShape.ctrlPtnameLine = ctrlPtnameLine;
			canvasShape.ctrlPtnameShape = ctrlPtnameShape;
			canvasShape.ctrlPtgroup = ctrlPtgroup;
			
			$previewShape.drawLine({
						layer:true,
						name:ctrlPtnameLine,
						group:ctrlPtgroup,
  						strokeStyle: "#000",
  						strokeWidth: 1,
  						x1: linePos, y1: 0,
  						x2: linePos, y2: lineSize
			});
			//TODO: add more shape
			switch (ctrlPt.shape){
				case "circle" :
				case  "ellipse"	:
					$previewShape.drawEllipse({
						layer: true,
						name:ctrlPtnameShape,
						group:ctrlPtgroup,
						fillStyle: colorShape,
  						x: xShape, y: yShape,
  						width: yShape*0.6, height: yShape*0.6
  				});
  				break;
  				case "rectangle":
  				case "square":
					$previewShape.drawRect({
  						layer: true,
  						name:ctrlPtnameShape,
  						group:ctrlPtgroup,
  						fillStyle: colorShape,
  						x: xShape, y: yShape,
  						width: yShape*0.6, height: yShape*0.6
				});
				break;
				case "roundrectangle":
					$previewShape.drawRect({
  						layer: true,
  						name:ctrlPtnameShape,
  						group:ctrlPtgroup,
  						fillStyle: colorShape,
  						x: xShape, y: yShape,
  						width: yShape*0.6, height: yShape*0.6,
  						cornerRadius: 3
				});
				break;	
				case "triangle":
					$previewShape.drawPolygon({
						layer: true,
  						name:ctrlPtnameShape,
  						group:ctrlPtgroup,
  						strokeStyle: colorShape,
  						fillStyle: colorShape,
  						x: xShape, y: yShape+3,
  						radius: 9,
 						sides: 3
				});
				break;
				case "solid":
					$previewShape.drawLine({
						layer:true,
						name:ctrlPtnameShape,
						group:ctrlPtgroup,
  						strokeStyle: colorShape,
  						strokeWidth: 2,
  						x1: xShape, y1: yShape,
  						x2: xShape+20, y2: yShape
				});
				break;
				case "dashed":
					$previewShape.drawLine({
						layer:true,
						name:ctrlPtnameShape,
						group:ctrlPtgroup,
  						strokeStyle: colorShape,
  						strokeWidth: 2,
  						strokeDash: [5,3],
  						x1: xShape, y1: yShape,
  						x2: xShape+20, y2: yShape
				});	
				break;
				case "dotted":
					$previewShape.drawLine({
						layer:true,
						name:ctrlPtnameShape,
						group:ctrlPtgroup,
  						strokeStyle: colorShape,
  						strokeWidth: 4,
  						rounded: true,
  						strokeDash: [1,6],
  						x1: xShape, y1: yShape,
  						x2: xShape+20, y2: yShape
					});	
				break;
				case "tee":
					$previewShape.drawRect({
  						layer: true,
  						name:ctrlPtnameShape,
  						group:ctrlPtgroup,
  						fillStyle: colorShape,
  						x: xShape, y: yShape,
  						width: yShape/4, height: yShape/2
				});
				break;
				case "diamiond":
			    	$previewShape.draw({
			    		layer:true,
						name:ctrlPtnameShape,
						group:ctrlPtgroup,
						fn: function(ctx) {
   					 		ctx.beginPath();
   					 		ctx.moveTo(xShape,yShape);
							ctx.lineTo(xShape+8,yShape-8);
							ctx.lineTo(xShape+16,yShape);
							ctx.lineTo(xShape+8,yShape+8);
   					 		ctx.fillStyle = colorShape;
    						ctx.stroke();
    						ctx.closePath();
							ctx.fill();
  					}
  				
					});
			    break;	
				case "pentagon":
			    	$previewShape.drawPolygon({
						layer: true,
  						name:ctrlPtnameShape,
  						group:ctrlPtgroup,
  						strokeStyle: colorShape,
  						fillStyle: colorShape,
  						x: xShape, y: yShape,
  						radius: 8,
 						sides: 5,
  				
					});
			    break;
			    case "hexagon":
			    	$previewShape.drawPolygon({
						layer: true,
  						name:ctrlPtnameShape,
  						group:ctrlPtgroup,
  						strokeStyle: colorShape,
  						fillStyle: colorShape,
  						x: xShape, y: yShape,
  						radius: 8,
 						sides: 6,
  				
					});
			    break;
			    case "heptagon":
			    	$previewShape.drawPolygon({
						layer: true,
  						name:ctrlPtnameShape,
  						group:ctrlPtgroup,
  						strokeStyle: colorShape,
  						fillStyle: colorShape,
  						x: xShape, y: yShape,
  						radius: 8,
 						sides: 7,
  				
					});
			    break;
			    case "octagon":
			    	$previewShape.drawPolygon({
						layer: true,
  						name:ctrlPtnameShape,
  						group:ctrlPtgroup,
  						strokeStyle: colorShape,
  						fillStyle: colorShape,
  						x: xShape, y: yShape,
  						radius: 8,
 						sides: 8,
  				
					});
			    break;					
			};
			return canvasShape;
		},
		
		/**
		 * Get the diff position to ordered controls points.
		 * @method ctrlPtComparator
		 * @param {int} l the left position
		 * @param {int} r the right position
		 * @private
		 */
		ctrlPtComparator : function(l,r) {
			return l.position - r.position;
		},
		
		/**
		 * Apply style properties (color, shape, opacity, size) to a distribution of control points 
		 * with gradient mode for color and size. 
		 * @method vizSwitcherGrad
		 * @param {object} $elAttr JQuery Selector
		 * @param {int} value0 a value of the distribution (ex: Sequence length, coverage).
		 * @param {int} value1 the next value of the distribution.
		 * @param {String} color0 the color associated with a value of distribution.
		 * @param {String} color1 the color associated with the next value of distribution.
		 * @param {String} shape the shape associated with a value of distribution.
		 * @param {int} opacity the opacity associated with a value of distribution.
		 * @param {int} size0 the size associated with a value of distribution.
		 * @param {int} size1 the size associated with the next value of distribution.
		 * @param {String} propertyName the type of distribution (length or coverage).
		 * @param {boolean} holdRoot define if the node is locked or not.
		 * @private
		 */
		//TODO: Skip this function if resizing
		vizSwitcherGrad : function  ($elAttr, value0, value1, color0, color1, shape , opacity, size0, size1, propertyName, holdRoot) {

				var mapdataColor = "mapData("+propertyName+","+value0+","+value1+","+color0+","+color1+")",
					mapdataSize = "mapData("+propertyName+","+value0+","+value1+","+size0+","+size1+")";
					//console.log("vizSwitcherGrad")
	
				switch($elAttr){
 						case "nodes_fill" :
 						case "nodes_colors": //pre-configured
 						case "nodes_shapes": //preconfigured
							this.nodesFill (value0,  mapdataColor, shape , opacity,  mapdataSize, propertyName, holdRoot);	
						break;
						case "nodes_border" :	
							this.nodesBorder (value0,  mapdataColor, shape , opacity,  mapdataSize, propertyName, holdRoot);	
						break;
						case "nodes_label" :	
							this.nodesLabel(value0,  mapdataColor , opacity,  mapdataSize, propertyName, holdRoot);	
						break;
						case "edges_line" :
						case "edges_shapes": //preconfigured
						case "edges_colors": //pre-configured		
							this.edgesLine (value0,  mapdataColor, shape , opacity,  mapdataSize, propertyName);	
						break;
						case "edges_target" :	
							this.edgesTarget (value0,  mapdataColor, shape, opacity,  propertyName);	
						break;
						case "edges_source" :	
							this.edgesSource (value0,  mapdataColor, shape, opacity, propertyName);	
						break;
						case "edges_label" :	
							this.egdesLabel (value0,  mapdataColor, shape , opacity,  mapdataSize, propertyName);	
						break;
						
					};
			
		},
		
	   /**
		 * Apply style properties (color, shape, opacity, size) to a distribution of control points 
		 * with discrete mode for color and size. 
		 * @method vizSwitcherDisc
		 * @param {object} $elAttr JQuery Selector.
		 * @param {int} value a value of the distribution (ex: Sequence length, coverage).
		 * @param {String} color the color associated with a value of distribution.
		 * @param {String} shape the shape associated with a value of distribution.
		 * @param {int} opacity the opacity associated with a value of distribution.
		 * @param {int} size the size associated with a value of distribution.
		 * @param {String} propertyName the type of distribution (length or coverage).
		 * @param {boolean} holdRoot define if the node is locked or not.
		 * @private
		 */
		//TODO: Skip this function if resizing
	    vizSwitcherDisc : function ($elAttr, value, color, shape , opacity, size, propertyName, holdRoot) {
				switch($elAttr){
 						case "nodes_fill" :
 						case "nodes_colors": //pre-configured
 						case "nodes_shapes": //preconfigured
							this.nodesFill (value,  color, shape , opacity, size, propertyName, holdRoot);	
						break;
						case "nodes_border" :	
							this.nodesBorder (value,  color, shape , opacity, size, propertyName, holdRoot);	
						break;
						case "nodes_label" :	
							this.nodesLabel (value,  color , opacity, size, propertyName, holdRoot);	
						break;
						case "edges_line" :
						case "edges_shapes": //preconfigured
						case "edges_colors": //pre-configured	
							this.edgesLine (value,  color, shape , opacity, size, propertyName);	
						break;
						case "edges_target" :	
							this.edgesTarget (value,  color, shape, opacity, propertyName);	
						break;
						case "edges_source" :	
							this.edgesSource (value,  color, shape, opacity, propertyName);	
						break;
						case "edges_label" :	
							this.egdesLabel (value,  color, shape , opacity, size, propertyName);	
						break;
					};
				
			
		},
		
		/**
		 * Apply style properties to nodes fill.
		 * @method nodesFill
		 * @param {int} value a value of the distribution (ex: Sequence length, coverage).
		 * @param {String} color the color associated with a value of distribution.
		 * @param {String} shape the shape associated with a value of distribution.
		 * @param {int} opacity the opacity associated with a value of distribution.
		 * @param {int} size the size associated with a value of distribution.
		 * @param {String} propertyName the type of distribution (length or coverage).
		 * @param {boolean} holdRoot define if the node is locked or not.
		 * @private
		 */
		nodesFill : function (value,  color, shape , opacity, size, propertyName, holdRoot) {
					var mapNodes = cy.nodes("["+propertyName+">="+value+"]");
						if(mapNodes.length != 0){	
						
							mapNodes.css({
										"width": size,
										"height": size,
										"shape": shape,
										"backgroundColor" : color,
										"backgroundOpacity" : opacity
								});	
						}
						
						if(holdRoot && cy._private.root!=undefined){
							
							//hold node root fill style
							var root = cy._private.root;
							var previousCss = root.data("previousCss");
							root.css({
										"width": previousCss.width,
										"height": previousCss.height,
										"shape": previousCss.shape,
										"backgroundColor" : previousCss.backgroundColor,
										"backgroundOpacity" : previousCss.backgroundOpacity
							});
						};
						
						
		},
		
		/**
		 * Apply style properties to nodes borders.
		 * @method nodesBorder
		 * @param {int} value a value of the distribution (ex: Sequence length, coverage).
		 * @param {string} color the color associated with a value of distribution.
		 * @param {String} shape the shape associated with a value of distribution.
		 * @param {int} opacity the opacity associated with a value of distribution.
		 * @param {int} size the size associated with a value of distribution.
		 * @param {String} propertyName the type of distribution (length or coverage).
		 * @param {boolean} holdRoot define if the node is locked or not.
		 * @private
		 */
		nodesBorder: function (value,  color, shape , opacity, size, propertyName, holdRoot) {
						var mapNodes = cy.nodes("["+propertyName+">="+value+"]");
						if(mapNodes.length != 0){
							mapNodes.css({
										"border-color" :color,
										"border-opacity" : opacity
										//"weight": array_size
									});
						};
												
						if(holdRoot && cy._private.root!=undefined){
							//hold node root border style
							var root = cy._private.root;
							var previousCss = root.data("previousCss");
							root.css({
										"borderColor" :previousCss.borderColor,
										"borderOpacity" : previousCss.Opacity,
										//weight: previousCss.weight
										
							});
						};
		},

	 	/**
		 * Apply style properties to nodes labels.
		 * @method nodesLabel
		 * @param {int} value a value of the distribution (ex: Sequence length, coverage).
		 * @param {String} color the color associated with a value of distribution.
		 * @param {int} opacity the opacity associated with a value of distribution.
		 * @param {int} size the size associated with a value of distribution.
		 * @param {String} propertyName the type of distribution (length or coverage).
		 * @param {boolean} holdRoot define if the node is locked or not.
		 * @private
		 */
	 	nodesLabel : function (value,  color , opacity, size, propertyName, holdRoot) {
						var mapNodes = cy.nodes("["+propertyName+">="+value+"] " );
						if(mapNodes.length != 0){
							mapNodes.css({
										"color" :color,
										"label-opacity" : opacity,
										//"font-size": array_size
									});
						}
						
						if(holdRoot && cy._private.root!=undefined){
							//hold node root label style
							var root = cy._private.root;
							var previousCss = root.data("previousCss");
							root.css({
										"color" :previousCss.color,
										"labelOpacity" : previousCss.labelOpacity,
										//fontSize: previousCss.fontSize,
										
							});
						}
						
		},
		/**
		 * Apply style properties to edges lines.
		 * @method nodesLabel
		 * @param {int} value a value of the distribution (ex: Sequence length, coverage).
		 * @param {String} color the color associated with a value of distribution.
		 * @param {String} shape the shape associated with a value of distribution.
		 * @param {int} opacity the opacity associated with a value of distribution.
		 * @param {int} size the size associated with a value of distribution.
		 * @param {String} propertyName the type of distribution (length or coverage).
		 * @private
		 */
		edgesLine: function(value,  color, shape , opacity, size, propertyName) {
						var mapEdges = cy.edges("["+propertyName+">="+value+"]");
						if(mapEdges.length != 0){
							mapEdges.css({
										"line-style": shape,
										"line-color" :color,
										"opacity" : opacity
										//"width": array_size,
							});
						};

		},

		/**
		 * Apply style properties to edges target.
		 * @method nodesLabel
		 * @param {int} value a value of the distribution (ex: Sequence length, coverage).
		 * @param {String} color the color associated with a value of distribution.
		 * @param {String} shape the shape associated with a value of distribution.
		 * @param {int} opacity the opacity associated with a value of distribution.
		 * @param {String} propertyName the type of distribution (length or coverage).
		 * @private
		 */
		edgesTarget: function (value,  color, shape, opacity, propertyName) {
						var mapEdges = cy.edges("["+propertyName+">="+value+"]" );
						if(mapEdges.length != 0){
							mapEdges.css({
										"target-arrow-shape": shape,
										"target-arrow-color" :color,
										"opacity" : opacity
							});
						};
		},

		/**
		 * Apply style properties to edges source.
		 * @method nodesLabel
		 * @param {int} value a value of the distribution (ex: Sequence length, coverage).
		 * @param {String} color the color associated with a value of distribution.
		 * @param {String} shape the shape associated with a value of distribution.
		 * @param {int} opacity the opacity associated with a value of distribution.
		 * @param {String} propertyName the type of distribution (length or coverage).
		 * @private
		 */
		edgesSource: function (value,  color, shape, opacity, propertyName) {
						var mapEdges = cy.edges("["+propertyName+">="+value+"]" );
						if(mapEdges.length != 0){
							var previousCss = mapEdges.data("previousCss");
							mapEdges.css({
										"source-arrow-shape": shape,
										"source-arrow-color" :color,
										"opacity" : opacity
							});
						};
		},

		/**
		 * Apply style properties to edges labels
		 * @method nodesLabel
		 * @param {int} value a value of the distribution (ex: Sequence length, coverage).
		 * @param {String} color the color associated with a value of distribution.
		 * @param {String} shape the shape associated with a value of distribution.
		 * @param {int} opacity the opacity associated with a value of distribution.
		 * @param {int} size the size associated with a value of distribution.
		 * @param {String} propertyName the type of distribution (length or coverage).
		 * @private
		 */
		egdesLabel: function(value,  color, shape , opacity, size, propertyName) {
						var mapEdges = cy.edges("["+propertyName+">="+value+"]" );
						if(mapEdges.length != 0){
							var previousCss = mapEdges.data("previousCss");
							mapEdges.css({
									"color" : color,
									"label-opacity" : opacity,
									//"font-size": array_size,
							});
						}

		}
	
	};

	
	/**
	 * ControlPoint define a class object and add html elements for each control point created.
	 * @class ControlPoint
	 * @param {object} $parentEl the JQuery selector.
	 * @param {object} initialState the initials prperties values (color, shape, size...).
	 * @param {String} orientation the orientation of the VizMapselection (not implemented yet).
	 * @param {object} listener reference to the VizMapSelection object. 
	 * @param {object} ctrlPtConfig the ctrlPtConfig object associated to this control point.
	 * @constructor
	 */
	function ControlPoint($parentEl, initialState, orientation, listener, ctrlPtConfig) {
		var self = this;
		self.$el = $("<div class='vizmapperPicker-ctrlPt'></div>");
		$parentEl.append(self.$el);
		self.$parentEl = $parentEl;
	
		self.attrCss = listener.attrCss;
		self.configView = ctrlPtConfig;
		self.listener = listener;
		
		var $elAttr = $parentEl.parent().attr("property");
		var shape = self.shape;
		
		//Initialize properties values
		if (typeof initialState === "string") {
			initialState = initialState.split(" ");
			self.position = parseFloat(initialState[1])/100;
			self.renderPos = self.position * listener.g2d.canvas.width;
			self.color = initialState[0];
			self.opacity = initialState[2];
			self.shape = initialState[3];
			self.size = parseInt(initialState[4]);
			//self.type = initialState[5];
		} else {
			self.position = initialState.position;
			self.renderPos = initialState.renderPos;
			self.color = initialState.color;
			self.shape = initialState.shape;
			self.opacity = initialState.opacity;
			self.size = parseInt(initialState.size);
			//self.type = initialState.type;
		}
		
		
		self.outerWidth = self.$el.outerWidth();
		//Define bacjkground color in function of type of vizMapSelection( color or shape)
		if(this.listener.opts.advanced || this.listener.attrCss=="colors"){
			self.$el.css("background-color", self.color);
		}else {
			self.$el.css("background-color", "#000");
		}
		
		self.$el.attr("opacity", self.opacity);
		self.$el.attr("shape", self.shape);
		self.$el.attr("size", self.size);
		
		//Define size in function of orientation
		if (orientation == "horizontal") {
			var pxLeft = ($parentEl.width() - self.$el.outerWidth()) * (self.position);
			self.$el.css("left", pxLeft);

		}/*else {
			var pxTop = ($parentEl.height() - self.$el.outerHeight()) * (self.position);
			self.$el.css("top", pxTop);
		}*/
		
		/*
		 * Define and add values (length or coverage) under the control point
		 * The value is calculated with maximum length or coverage available in cy object in _private attribute
		 */
		self.property=$parentEl.parents().eq(1).attr("property").toLowerCase();
		if(self.listener.opts.group=="nodes"){	
			if(self.property=="length"){
				self.lengthSeq = parseInt(self.position * (cy._private.maxNodesLength+1)) ;
				var posCtrlPtLen = '<a class=ui-sel_pos>' +self.lengthSeq+ '</a>';
				self.$posCtrlPtLen = $(posCtrlPtLen);
				self.$el.append(self.$posCtrlPtLen);
			}else if(self.property=="coverage"){
				self.coverage = parseInt(self.position * (cy._private.maxNodesCoverage+1)) ;
				var posCtrlPtCoverage = '<a class=ui-sel_pos>' +self.coverage+ '</a>';
				self.$posCtrlPtCoverage = $(posCtrlPtCoverage);
				self.$el.append(self.$posCtrlPtCoverage);
			}
		}else if(self.listener.opts.group=="edges" ){
			if(self.property=="coverage"){
				self.coverage = parseInt(self.position * (cy._private.maxEdgesCoverage+1)) ;
				var posCtrlPtCoverage = '<a class=ui-sel_pos>' +self.coverage+ '</a>';
				self.$posCtrlPtCoverage = $(posCtrlPtCoverage);
				self.$el.append(self.$posCtrlPtCoverage);
			}
		} 

		//Define event 
		self.drag = bind(self.drag, self);
		self.stop = bind(self.stop, self);
		self.clicked = bind(self.clicked, self);
		self.colorChanged = bind(self.colorChanged, self);
		self.shapeChanged = bind(self.shapeChanged, self);
		self.opacityChanged = bind(self.opacityChanged, self);
		self.sizeChanged = bind(self.sizeChanged, self);
		//self.typeChanged = bind(self.typeChanged, self);
		self.posChanged = bind(self.posChanged, self);
		self.$el.draggable({
			axis: (orientation == "horizontal") ? "x" : "y",
			drag: self.drag,
			stop: self.stop,
			containment: $parentEl
		});
		
		self.$el.click(self.clicked);
		self.$el.data(self);
		
		//Draw shape in canvas
		//TODO: test with type (color, shape) in vizMapSelection
		var $previewShape = self.$parentEl.parent().find(".vizmapperPicker-preview-shape");
		self.canvasShape = listener.drawPreviewShape($previewShape, self);

		/*setTimeout(function() {
			self.typeChanged();
		},50);*/
	
	}

	ControlPoint.prototype = {
		/**
		 * Move a control point.
		 * @event drag
		 * @param {object} e this event.
		 * @param {object} ui object contained the position of the control point.
		 */
		drag: function(e, ui) {
			// convert position to a %
			var left = ui.position.left;
			var strPosProperty;
			this.position = (left / (this.$parentEl.width() - this.outerWidth));
			this.renderPos = left;
			
			//update sequence length on drag
			if(this.listener.opts.group=="nodes"){	
				if(this.property=="length" && this.listener.opts.group=="nodes"){
					this.lengthSeq = parseInt(this.position * (cy._private.maxNodesLength+1));
					this.$posCtrlPtLen.text(this.lengthSeq);
					strPosProperty = JSON.stringify(this.lengthSeq);				
				}else if(this.property=="coverage"){
					this.coverage = parseInt(this.position * (cy._private.maxNodesCoverage+1));
					this.$posCtrlPtCoverage.text(this.coverage);
					strPosProperty = JSON.stringify(this.coverage);
				}
			}else if(this.listener.opts.group=="edges"){
				if(this.property=="coverage"){
					this.coverage = parseInt(this.position * (cy._private.maxEdgesCoverage+1));
					this.$posCtrlPtCoverage.text(this.coverage);
					strPosProperty = JSON.stringify(this.coverage);
				}
			}
			
			//move the separator line and the shape on the preview on drag
			if(this.attrCss == "shapes"){
				var linePos = this.position * this.canvasShape.$previewShape.width();
				var lineSize = this.canvasShape.$previewShape.height();
				var xShape = linePos+12;
				var yShape = lineSize/2;
				var canvasShape = this.canvasShape;
				var $previewShape = canvasShape.$previewShape;
				
				var ctrlPtgroup = "ctrlPt"+strPosProperty+"Group";
				var colorShape = this.color;
				var currentShape = this.shape;
							
				$previewShape.setLayer(this.canvasShape.ctrlPtnameLine, {
	  				x1: linePos, y1: 0,
	  				x2: linePos, y2: lineSize
				}).drawLayers();
				
				switch (currentShape){
					case "ellipse":
					case "circle":
					case "triangle":
					case "square":
					case "rectangle":
					case "roundrectangle":
					case "diamond":
					case "pentagon":
					case "heptagon":
					case "hexagon":
					case "octagon":
						$previewShape.setLayer(this.canvasShape.ctrlPtnameShape, {
	  						x: xShape, y: lineSize/2,
						}).drawLayers();
					break;
					case 'solid':
					case 'dashed':
					case 'dotted':
						$previewShape.setLayer(this.canvasShape.ctrlPtnameShape, {
		  					x1: xShape, y1: yShape,
		  					x2: xShape+20, y2: yShape
						}).drawLayers();
					break;
				
				}
			}
			
			this.listener.updatePreview();
		},

		/**
		 * Update view when stop the move of control point.
		 * @event Stop
		 */
		stop: function() {
			this.listener.updateView();
		},

		/**
		 * Show the control point config after click on control point.
		 * @event clicked
		 */
		clicked: function() {
			this.configView.show(this.$el.position(), this.color, this.shape, this.opacity, this.size, this);
			this.listener.updatePreview();
		},

		/**
		 * Set the color and update preview.
		 * @event colorChanged
		 * @param {String} c the new color. 
		 */
		colorChanged: function(c) {
			this.color = c;
			this.$el.css("background-color", c);
			
			/*this.canvasShape.$previewShape.setLayer(this.canvasShape.ctrlPtnameShape, {
  							fillStyle: c,
							strokeStyle: c,
			}).drawLayers();*/
			
			this.listener.updatePreview();
			
		},

		/**
		 * Set the shape and update preview and view.
		 * @event shapeChanged
		 * @param {String} s the new shape.
		 */
		shapeChanged: function(s) {
			this.shape = s;
			this.$el.attr("shape", this.shape);
			
			var canvasShape = this.canvasShape;
			var $previewShape = canvasShape.$previewShape;
			
			this.canvasShape.$previewShape.removeLayerGroup(this.canvasShape.ctrlPtgroup).drawLayers();
			this.canvasShape = this.listener.drawPreviewShape($previewShape, this);
			this.listener.updateView();
		},

		/**
		 * Set the opacity and update preview.
		 * @event opacityChanged
		 * @param {String} o the new opacity. 
		 */
		opacityChanged: function(o) {
			this.opacity = o;
			this.$el.attr("opacity", o);
			this.listener.updatePreview();
		},

		/**
		 * Set the opacity and update preview and view.
		 * @event sizeChanged
		 * @param {String} w the new size.
		 */
		sizeChanged: function(w) {
			this.size = w;
			this.$el.attr("size", w);
			this.listener.updateView();
		},
		
		/*typeChanged: function(t) {
			this.type = t;
		},*/
		
		/**
		 * Set the position without drag when VizMapSelection is resize (use with only when JQuery accordion is active).
		 * Bug with position calculated!!!
		 * @beta 
		 * @method sizeChanged
		 * @depreciate 
		 * @param {String} p the new position.
		 * 
		 */
		posChanged: function(p){
			var canvasShape = this.canvasShape;
			var $previewShape = canvasShape.$previewShape;
			var widthCorrection = $(".ui-continuous_map > canvas").get(0).width;
			var $continuousMap = this.$parentEl.parent();
			canvasShape.$previewShape.css("width", widthCorrection);
			this.renderPos = p;

			//TODO: fix bug set position after resize
			//this.position = (p / (parseFloat($(".ui-continuous_map").css("width")) - this.outerWidth));
			this.canvasShape.$previewShape.removeLayerGroup(this.canvasShape.ctrlPtgroup).drawLayers();
			this.canvasShape = this.listener.drawPreviewShape($previewShape, this);
			this.listener.updatePreview();
		},

		/**
		 * Delete the control point and update view and preview.
		 * @event removeClicked
		 */
		removeClicked: function() {
			this.listener.removeControlPoint(this);

			if(this.attrCss == "shapes"){
				this.canvasShape.$previewShape.removeLayerGroup(this.canvasShape.ctrlPtgroup).drawLayers();
			}
			
			this.listener.updatePreview();
		},
		
		
	};
	
	/**
	 * ControlPtConfig define a class object and add html elements for each instance of ViLMapSelection.
	 * This object allow to configure or delete controls points:
	 * - on click picker is called to display a bubble pickers/setters for set properties (color, shape, ...) of a control point. 
	 * - on click on crux remove the control point. 
	 * Initially, the html element is hidden, there has been shown after click on a control point.
	 * 
	 * @class ControlPtConfig
	 * @param {object} $parent the JQuery selector.
	 * @param {object} opts the object contained the initials values of attributes.
	 * @param {array} attrCss the list of type of vizmapper (Ex: ["color", "shape"]).
	 * @constructor
	 */
	function ControlPtConfig($parent, opts, attrCss) {
		var self = this;

		self.attrCss = attrCss;
		//add html elements
		self.$el = $('<div class="vizmapperPicker-ptConfig" style="visibility: hidden"></div>');
		$parent.append(self.$el);
		
		//add div element with right class for picker
		var $cpicker;
		if(!opts.advanced && $parent.attr('property')=="nodes_shapes"){
			$cpicker = $('<div class="shape-chooser"></div>');
		}else{
			$cpicker = $('<div class="color-chooser"></div>');
		}
		
		//add div with crux for delete control point
		self.$el.append($cpicker);
		var $rmEl = $("<div class='vizmapperPicker-close'></div>");
		self.$el.append($rmEl);
		 	
		$cpicker.addClass("continuous_map "+ $parent.attr('property')).attr("property", $parent.parent().attr('property'));
		
		if(opts.advanced){
			$cpicker.addClass("advanced");
		}
		
		//Define event
		self.colorChanged = bind(self.colorChanged, self);
		self.shapeChanged = bind(self.shapeChanged, self);
		self.opacityChanged = bind(self.opacityChanged, self);
		self.sizeChanged = bind(self.sizeChanged, self);
		//self.typeChanged = bind(self.typeChanged, self);
		self.removeClicked = bind(self.removeClicked, self);
		
		//Advanced chooser (color, shape and size)
		if(opts.advanced){
			$cpicker.picker({
				onShow: function (colpkr) {
								var $colpkr = $(colpkr);
								cy.elements(":selected").unselect();
								$colpkr.addClass(self.$el.attr('class')).removeClass('color-chooser continuous_map').attr('property',self.$el.attr('property'));								
								$colpkr.addClass("advanced");
								//cy.off("mouseout mouseover touchout touchover");
				},
				onSubmit : function () {
							self.shapeChanged(this.data("picker").shape);
							self.sizeChanged(this.data("picker").size);
							self.colorChanged(this.data("picker").color);
				},
				onChangeSize:function(){
						self.sizeChanged(this.data("picker").size);
				},
				onChangeColor: self.colorChanged,
				onHide: function () {
								self.listener.listener.updateView();
				},
				onChangeShape: function () {						
								self.shapeChanged(this.data("picker").shape);			
				}


			});
		}else{	
			
			if(attrCss == "shapes"){
	            //shape-chooser	
	            $cpicker.picker({
						onShow: function (shapkr) {
								cy.elements(":selected").unselect();
								//cy.off("mouseout mouseover touchout touchover");
						},
						onChangeShape: function () {						
								self.shapeChanged(this.data("picker").shape);			
						},
						onChangeSize:function () {
									self.sizeChanged(this.data("picker").size);			
						},
						onHide: function () {
								self.listener.listener.updateView();
						},
						onSubmit : function () {
								self.shapeChanged(this.data("picker").shape);
								self.sizeChanged(this.data("picker").size);	
						},
						
		
		
					});
			}
			
			if(attrCss == "colors"){
					//color-chooser
					$cpicker.picker({
						onShow: function (colpkr) {
									cy.elements(":selected").unselect();
									//cy.off("mouseout mouseover touchout touchover");
						},
						onSubmit : function () {
									self.listener.listener.updateView();
						},
						onChangeColor: self.colorChanged,
						onHide: function () {
										self.listener.listener.updateView();
						}
				});
			}
		}
		

		self.$cpicker = $cpicker;
		self.opts = opts;
		self.visible = false;
		
		$rmEl.click(self.removeClicked);
		
		self.$el.data("ControlPtConfig", self);
		
	}

	ControlPtConfig.prototype = {
		/**
		 * Show ControlPtConfig and bubble pickers/setters.
		 * @param {int} position the position of the control point.
		 * @param {String} color the color of the control point.
		 * @param {String} shape the shape that the control point defined.
		 * @param {String} opacity the opacity of the control point.
		 * param {String} size the size define by the control point.
		 * @param {object} listener the parent object. 
		 * @event show
		 */
		show: function(position, color, shape, opacity, size,listener) {
			
			this.$cpicker.unbind("click");
			this.visible = true;
			this.listener = listener;
			this.$el.css("visibility", "visible");
			this.$cpicker.pickerSetValues(color, opacity, size, undefined, undefined, shape);
			this.$cpicker.css("background-color", "transparent");

			
			this.$cpicker.css("opacity", opacity);
			if (this.opts.orientation === "horizontal") {
				this.$el.css("left", position.left);
			}
			//vertical orientation is not implemented yet
			/*else {
				this.$el.css("top", position.top);
			}*/

			this.$cpicker.pickerShow();
		},
		/**
		 * Hide ControlPtConfig and bubble pickers/setters.
		 * @event hide
		 */
		hide: function() {
			if (this.visible) {
				this.$el.css("visibility", "hidden");
				this.visible = false;
			}
		},
		
		/**
		 * Call colorChanged of vizMapSelection. 
		 * @event colorChanged
		 * @param {array} hsb the new color in hsb
		 * @param {String} hex the new color in hex
		 * @param {array} rgb the new color in rgb
		 */
		colorChanged: function(hsb, hex, rgb) {
			hex = "#" + hex;
			this.listener.colorChanged(hex);
			//this.$cpicker.css("background-color", hex);
		},

		/**
		 * Call shapeChanged of vizMapSelection. 
		 * @event shapeChanged
		 * @param {String} s the new shape. 
		 */
		shapeChanged: function(s) {
			this.listener.shapeChanged(s);
		},

		/**
		 * Call opacityChanged of vizMapSelection. 
		 * @event opacityChanged
		 * @param {int} o the new opacity. 
		 */
		opacityChanged: function(o) {
			this.listener.opacityChanged(o);
		},
		/**
		 * Call sizeChanged of vizMapSelection. 
		 * @event sizeChanged
		 * @param {int} w the new size.
		 */
		sizeChanged: function(w) {
			this.listener.sizeChanged(w);
		},
		
		/*typeChanged: function(t) {
			this.listener.typeChanged(t);
			this.listener.type = t;
		},*/

		/**
		 * Call removeClicked of vizMapSelection.
		 * @event vizMapSelection
		 */
		removeClicked: function() {
			this.listener.removeClicked();
			this.hide();
		}
	};


	var methods = {
		
		/**
		 * Initialize a vizmapper instance.
		 * @method init
		 * @for VizMap
		 * @param {object} opts the object contained the values to initialize the class.
		 * @param {object} config the configuration file parsed. 
		 */
		init: function(opts, config) {	
			opts = $.extend({
					group: ["nodes"],
					property: ["length"],
					controlPoints:["#FFF 0% 1 ellipse 30", "#000 100% 1 ellipse 30"],
					accordion:config.vizmapper.with_accordion,
					orientation: "horizontal",
					center: false,
					attrCss: ["color"],
					type: "gradient",
					tableOfNodes:undefined,
					tableOfEdges:undefined,
					sequenceHeader : undefined,
					advanced:config.vizmapper.advanced,
					holdRoot : config.vizmapper.with_hold_root,
					typeChooser:config.vizmapper.with_type_choose,
					resetButton:config.vizmapper.with_reset_button,
					centerButton : config.vizmapper.with_center_input
					//gradType: "linear",
					//change: function() {}
			}, opts);
	
			var initialOpts = opts;
			
			this.empty();
			if(this.hasClass("ui-accordion")){
				this.accordion('destroy');
				//force display scrollbar before construct vizmap
				this.parent().css("overflow", "scroll");
			};
			
			var vizmapData = {pickers : []};
	
			var data = {
						property: opts.property,
						mapdata: 	[opts],
			};				
			//vizmapData.pickers.push(data);
				
			//var arrayOpts = vizmapData.pickers[0].mapdata;
						
			var vizMap = new VizMap(this, opts);
			var propContainer = vizMap.propContainer;
			var propContainerLen = propContainer.length;
			var $parent = vizMap.$el.parent();
			var outerWidth;
			 
			for (var i=0; i<propContainerLen; i++){
		
				//var attrCssLen = propContainer[i].attrCss.length;
	
				//var size = propContainer[0].attrCss[0].$el.css("width");
				//for (var j=0; j<attrCssLen; j++){
						var attrCss = propContainer[i].attrCss;
						var vizSel = new VizMapSelection(attrCss, opts, propContainer[i].property,  vizMap);
						var ctrlPtLen = vizSel.controlPoints.length;
						//vizSel.$el.css("width", size);
						vizSel.initialOpts = opts; 
						vizSel.controlPoints[0].$el.draggable("destroy");
						vizSel.controlPoints[ctrlPtLen-1].$el.draggable("destroy");	
						outerWidth = vizSel.controlPoints[ctrlPtLen-1].outerWidth;			
						attrCss.VizMapSelection = vizSel;
						attrCss.VizMapSelection.$el.data("VizMapSelection", vizSel); 
						if(opts.advanced){
							if(attrCss.VizMapSelection.property == opts.property[0].toLowerCase()){
								attrCss.VizMapSelection.updateView();
							};
						}else{
							attrCss.VizMapSelection.updateView();
						}			
				//};	
			}
			
			
			//set display scrollbar to auto for remove it if necessary 
			
			 if(opts.with_accordion){
			 			 $parent.css("overflow-x", "hidden");
			 		 	 vizMap.adjustToAccordion(outerWidth, 100);
			 		 	 $parent.css("overflow-y", "auto");
			 }
			
				
	
			if(opts.typeChooser){
				
				this.prepend('<form>\
									<label class="type" for="discrete">Discrete</label>\
									<input type="radio"  class="vizmapper_type" name="typeof_vizmap" id="discrete" value="discrete">\
									<label class="type" for="gradient">Gradient</label>\
									<input type="radio"  class="vizmapper_type" name="typeof_vizmap" id="gradient" value="gradient">\
							 </form>');
		
				
				if(opts.type == "discrete"){
					$(this).find("#discrete").prop("checked", true);
				}else if(opts.type == "gradient"){
					$(this).find("#gradient").prop("checked", true);	
				};
				
				this.find('.vizmapper_type').click(function(){
					//console.log(vizMap.listener.initialOpts)
					var type = $(this).val();
					var updateOpts= {type:type};
					vizMap.updateOptions(updateOpts);
					
				});
			}
			
			if(opts.resetButton){
				this.append("<button id=reset_vizmapper>Reset vizmapper</button>");	   
				$("#reset_vizmapper").click(function() {
					//TODO:reset dans vizmap prototype
					$("#vizmapper.content").vizMapReset(initialOpts, config);
				});
			};
			
			if(opts.centerButton){
					this.append('<div class="property behaviourGraph nodes">\
									<input id=focusNode_Vizmapper type="checkbox" />\
									<label class="style">Center on selected node</label>\
								</div>');
								
					if(opts.center){
						$("#focusNode_Vizmapper").prop("checked", true);
						cy._private.centered = true;
					}else{
						$("#focusNode_Vizmapper").prop("checked", false);
						cy._private.centered = false;
					}
					
					$("#focusNode_Vizmapper").click(function(){
						
						if($("#focusNode_Vizmapper").prop("checked")){
							opts.center = true;
							cy._private.centered = true;
						}else{
							opts.center = false;
							cy._private.centered = false;
						}
						
						var newOpts= {center:opts.center};
						vizMap.updateOptions(newOpts);
					});
			};
			
			if(opts.group == "nodes"){
	
				this.append("<div id=root_behaviour class=behaviourGraph core>\
								<input id=hold_root type='checkbox' \>\
								<label class='style'>Hold root node style</label>\
							</div>");
							
				var holdRoot = opts.holdRoot;
				if(holdRoot){
					$("#hold_root").prop("checked", true);
				}else{
					$("#hold_root").prop("checked", false);
				}
				
				var self =this;
				$("#hold_root").click(function(){
					if(holdRoot){
						holdRoot = false;	
					}else{
						holdRoot = true;
					}
					var newOpts= {"holdRoot":holdRoot};
					vizMap.updateOptions(newOpts);
				});
			}else if(opts.group == "edges"){
				$(".behaviourGraph.nodes").hide();
			}
			
			//Test if coverage is in the array property if yes add table of coverage files
			var lenProperty = opts.property.length;
			for(var n=0; n<lenProperty; n++){
				if(opts.property[n].toLowerCase()=="coverage"){
					vizMap.addFileCoverageTable(opts);
					break;
				}
			}
			
			this.data("VizMap", vizMap);
		},
						
		/**
		 * Save a vizmapper instance.
		 * @method save
		 * @for VizMap
		 */
		save:function() {			
			var vizMap = this.data("VizMap");
			var mapper=[];
			var vizmapData = { mappers : mapper};
			var propContainer = vizMap.propContainer;
			var propContainerLen = propContainer.length;
			var initialOpts;
			for (var i=0; i<propContainerLen; i++){

					var attrCss = propContainer[i].attrCss;
					//var attrCssLen = propContainer[i].attrCss.length;
					
					//for (var j=0; j<attrCssLen; j++){
							var vizSel = attrCss.VizMapSelection;
							var opts = vizSel.opts;
							initialOpts = vizSel.initialOpts;
							var arrayCtrlPts = [];
							var controlPoints =  vizSel.controlPoints;
							var controlPointsLen = controlPoints.length;
							
						for (var k=0; k<controlPointsLen; k++){
							    var ctrlPrl = controlPoints[k];				
								var color = ctrlPrl.color ;
								var pos = ctrlPrl.position *100;
								var shape = ctrlPrl.shape;
								var opacity = ctrlPrl.opacity;
								var size = ctrlPrl.size;
								var ctrlPtStr= color+" "+pos+" "+opacity+" "+shape+" "+size;
								arrayCtrlPts[k]=ctrlPtStr;
						};
						
						
						var newOpts = {
								//change :opts.change ,
								controlPoints : arrayCtrlPts,
								holdRoot:opts.holdRoot,
								center:opts.center,
								group : opts.group,
								orientation: opts.orientation,
								attrCss : opts.attrCss,
								property : opts.property,
								type : opts.type,
								tableOfNodes:opts.tableOfNodes,
								tableOfEdges:opts.tableOfEdges,
								sequenceHeader : opts.sequenceHeader,
								advanced:opts.advanced,
								accordion:opts.accordion,
								typeChooser:opts.typeChooser,
								resetButton:opts.resetButton,
								centerButton : opts.centerButton
						
						};				

	
					//};	
				var data = {
							property: vizMap.propContainer[i].property,
							opts: 	newOpts,
							initialOpts: initialOpts
						};
						
		    	vizmapData.mappers.push(data);
				
			};

			return vizmapData;
					
		},
		
		/**
		 * REstore a vizmapper instance with a save of instance.
		 * @method restore
		 * @for VizMap
		 * @param {object} save a save of an instance of a vizmapper.
		 * @param {object} config the configuration file parsed. 
		 */
		restore:function(save , config){

			this.empty();
			
			var mappers = save.mappers;
			var defaultOpts = $.extend({
					group: "nodes",
					property: ["length"],
					controlPoints:["#FFF 0% 1 ellipse 30", "#000 100% 1 ellipse 30"],
					accordion:config.vizmapper.with_accordion,
					orientation: "horizontal",
					center: false,
					attrCss: ["color"],
					type: "gradient",
					tableOfNodes:undefined,
					tableOfEdges:undefined,
					sequenceHeader : undefined,
					advanced:config.vizmapper.advanced,
					holdRoot : config.vizmapper.with_hold_root,
					typeChooser:config.vizmapper.with_type_choose,
					resetButton:config.vizmapper.with_reset_button,
					centerButton : config.vizmapper.with_center_input
					//gradType: "linear",
					//change: function() {}
			}, mappers[0].opts);

			var vizMap = new VizMap(this,defaultOpts);	
			var propContainer = vizMap.propContainer;
			var propContainerLen = mappers.length;
			var $parent = vizMap.$el.parent();
			var outerWidth;
			var centerOpts = false;
			
			
			if(this.hasClass("ui-accordion")){
				this.accordion('destroy');
				//force display scrollbar before construct vizmap
				vizMap.$el.parent().css("overflow-y", "scroll");
			}
			
			//var h=0;
			for (var i=0; i<propContainerLen; i++){
					
				var opts = mappers[i].opts;
				var initialOpts = mappers[i].initialOpts;
				//var optsLen = arrayOpts.length;
				//for (var j=0; j<optsLen; j++ ){
						var attrCss = propContainer[i].attrCss;
						var vizSel = new VizMapSelection(attrCss, opts, propContainer[i].property, vizMap);
						var ctrlPtLen = vizSel.controlPoints.length;					
						vizSel.initialOpts = initialOpts;
						vizSel.controlPoints[0].$el.draggable("destroy");
						vizSel.controlPoints[ctrlPtLen-1].$el.draggable("destroy");	
						outerWidth = vizSel.controlPoints[ctrlPtLen-1].outerWidth;	
						attrCss.VizMapSelection = vizSel;
						attrCss.VizMapSelection.$el.data("VizMapSelection", vizSel);
						if(opts.advanced){
							if(attrCss.VizMapSelection.property == mappers[0].property){
									attrCss.VizMapSelection.updateView();
							};
						}else{
							attrCss.VizMapSelection.updateView();
						}
									
				//};
				
				//h++;			
			};
			
			//set display scrollbar to auto for remove it if necessary
			
			if(opts.accordion){
					$parent.css("overflow-x", "hidden"); 
				 	vizMap.adjustToAccordion(outerWidth, 1); 
				 	$parent.css("overflow-y", "auto");
			}
			
			if(opts.typeChooser){	
				this.prepend('<form>\
									<label class="type" for="discrete">Discrete</label>\
									<input type="radio"  class="vizmapper_type" name="typeof_vizmap" id="discrete" value="discrete">\
									<label class="type" for="gradient">Gradient</label>\
									<input type="radio"  class="vizmapper_type" name="typeof_vizmap" id="gradient" value="gradient">\
							 </form>');
				
		
				if(opts.type == "discrete"){
					$(this).find("#discrete").prop("checked", true);
				}else if(vizMap.opts.type == "gradient"){
					$(this).find("#gradient").prop("checked", true);	
				};
				
				this.find('.vizmapper_type').click(function(){
					var type = $(this).val();
					var opts= {type:type};
					vizMap.updateOptions(opts);
				});
			}
				
		
			if(opts.resetButton){
				this.append("<button id=reset_vizmapper>Reset vizmapper</button>");	   
				$("#reset_vizmapper").click(function() {
					//TODO:reset dans vizmap prototype
					$("#vizmapper.content").vizMapReset(initialOpts, config);
				});
			};
					
			if(opts.centerButton){
					
					this.append('<div class="property behaviourGraph nodes">\
									<input id=focusNode_Vizmapper type="checkbox" />\
									<label class="style">Center on selected node</label>\
								</div>');
					
					if(cy._private.centered){
						$("#focusNode_Vizmapper").prop("checked", true);
						opts.center = true;
					}else{
						$("#focusNode_Vizmapper").prop("checked", false);
						opts.center = false;
					}
					
				
					$("#focusNode_Vizmapper").click(function(){
						
						if($("#focusNode_Vizmapper").prop("checked")){
							opts.center = true;
							cy._private.centered = true;
						}else{
							opts.center = false;
							cy._private.centered = false;
						}
						
						var newOpts= {center:opts.center};
						vizMap.updateOptions(newOpts);
					});
			};
	
			if(opts.group == "nodes"){
	
				this.append("<div id=root_behaviour class=behaviourGraph core>\
								<input id=hold_root type='checkbox' \>\
								<label class='style'>Hold root node style</label>\
							</div>");
				
				var holdRoot = opts.holdRoot;
				
				if(holdRoot){
					$("#hold_root").prop("checked", true);
				}else{
					$("#hold_root").prop("checked", false);
				}
				
				var self =this;
				$("#hold_root").click(function(){
					var newOpts;
					if(holdRoot){
						holdRoot = false;	
					}else{
						holdRoot = true;
					}
					var newOpts= {"holdRoot":holdRoot};
					vizMap.updateOptions(newOpts);
				});
			}else if(opts.group == "edges"){
				$(".behaviourGraph.nodes").hide();
			}
			
			//Test if coverage is in the array property if yes add table of coverage files
			var lenProperty = mappers.length;
			for(var n=0; n<lenProperty; n++){
				if(mappers[n].property=="coverage"){
					vizMap.addFileCoverageTable(opts);
					break;
				}
			}
			this.data("VizMap", vizMap);
		},
		
		/**
		 * Update a vizmapper instance with new opts.
		 * @method update
		 * @for VizMap
		 * @param {object} opts the object with new values of attributes.
		 * @param {object} config the configuration file parsed. 
		 */
		//TODO: update VizMapSelection opts without generate new item;
		update:function(save, config){
			
			var mappers = save.mappers; 
			var vizMap = this.data("VizMap");
			var propContainer = vizMap.propContainer;
			var propContainerLen = mappers.length;
			var $parent = vizMap.$el.parent();
			var outerWidth;

			for (var i=0; i<propContainerLen; i++){
					
				var opts = mappers[i].opts;
				var initialOpts = mappers[i].initialOpts;
				var attrCss = propContainer[i].attrCss;
				attrCss.$el.find('canvas, div').remove();
				//var vizSel = attrCss.VizMapSelection;
				//vizSel.updateOptions(opts);
				var vizSel = new VizMapSelection(attrCss, opts, propContainer[i].property, vizMap);	
				var ctrlPtLen = vizSel.controlPoints.length;					
				vizSel.initialOpts = initialOpts;
				vizSel.controlPoints[0].$el.draggable("destroy");
				vizSel.controlPoints[ctrlPtLen-1].$el.draggable("destroy");	
				outerWidth = vizSel.controlPoints[ctrlPtLen-1].outerWidth;	
				attrCss.VizMapSelection = vizSel;
				attrCss.VizMapSelection.$el.data("VizMapSelection", vizSel);
				if(opts.advanced){
					if(attrCss.VizMapSelection.property == mappers[0].property){
							attrCss.VizMapSelection.updateView();
					};
				}else{
					attrCss.VizMapSelection.updateView();
				}		
			};
			
		
			this.data("VizMap", vizMap);
			
		},
		
		/*updateOnlyView:function(){
			var vizSelActif = this.find(".ui-accordion-content-active").children();
			vizSelActif.each(function(){
				//console.log($(this).data("VizMapSelection"))
				$(this).data("VizMapSelection").updateView();				
			});	
		}*/
		
		/**
		 * Reset a vizmapper instance.
		 * @method reset
		 * @for VizMap
		 * @param {object} opts the object with initial values of attributes.
		 * @param {object} config the configuration file parsed. 
		 */
		reset: function(opts, config){
			
			cy.elements(":selected").unselect();

			/*var initType ={
				type : cy._private.vizmapperType
			};
	
			$.extend(opts,  initType);*/
			
			if(this.get(0).scrollHeight > this.parent().get(0).clientHeight){
				this.parent().css("overflow", "scroll");
			}
			
			this.empty();
			
			if(this.hasClass("ui-accordion")){
				this.accordion('destroy');
			};
			
			this.vizMap(opts, config);
			
			this.parent().css("overflow", "auto");
			//this.parent().css("overflow-y", "auto");
			//this.vizMapUpdateOnlyView();
					
		},
		
		/**
		 * Return if vizmapper is scrollable ar not (scrollbar appareace) 
		 * @method scrollable
		 * @for VizMap
		 */
		scrollable: function(){
			
			this.data().VizMap.isScrollable();
			return this.data().VizMap.scrollable;
			
		}
			
		
	};
	
	/** extend jquery functions
	 * 
	 * @namespace
	 */
	$.fn.extend({
		vizMap: methods.init,
		vizMapSave: methods.save,
		vizMapRestore: methods.restore,
		vizMapUpdate: methods.update,
		//vizMapUpdateOnlyView: methods.updateOnlyView,
		vizMapScrollable: methods.scrollable,
		vizMapReset: methods.reset
	});
	
	
	
	/*$.fn.VizMap = function(method, opts) {
		if (typeof method === "string" && method !== "init") {
			methods[method].call(this, opts);	
		} else {
			opts = method;
			methods.init.call(this, opts);
		}
		
	};*/
	
})( jQuery );
