/**
 * @summary     cytoscape.vizStyle
 * @description Add style editor to cytoscape.js graph area.
 * @version     
 * @file         cytoscape.vizStyle.js
 * @author      Alexan Andrieux
 * @contact     <a href="mailto:alexan.andrieux@inria.fr"> Alexan Andrieux</a>
 * @category
 *
 * @copyright 
 *
 * This source file is free software, under AGLP License
 * 
 * This source file is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the license files for details.
 * 
 * For details please refer to: <a href="http://www.gnu.org/licenses/agpl-3.0.html"></a>
 */

/** 
 * Display a tab vizstyle in tab when it is activate in config file.
 * 
 * Vizstyle allow to change style of graph elements (border, size, shape, colors, ...) 
 * and style of graph area (background, selection box, ...).
 * @module vizstyle
 * 
 * @requires <a href='http://blog.jquery.com/2011/09/12/jquery-1-6-4-released/'>jQuery Core 1.6.4</a>
 * @dependency <script language="JavaScript" type="text/javascript" src="../biojs/dependencies/jquery/jquery-1.4.2.min.js"></script>
 * 
 * @requires <a href='http://cytoscape.github.io/cytoscape.js/'>cytoscape 2.0.4+</a>
 * 
 * @example
 *		$(selector).vizStyle(startStyle);  //where sartStyle is the type of vizStyle (core, node, edge)
 */
(function($){
"use strict";	
	/**
	 * 
	 * VizStyle give the possibilities to set the global style of the graph:
	 * - graph background color																					   
	 * - selection box style (fill, border) : core																	   
	 * - nodes style  (fill, border, label) :node																	   	
	 * - edges style (fill, border, label)	 : edges																   
	 * 																												   
	 *  
	 * @class vizStyle
	 * @param {string} startStyle the type of vizstyle (core, node, edges)
	 * @constructor
	 * 
	 */
	function VizStyle($el){
		this.$el = $el;
	};
	
	VizStyle.prototype = {
		
	/**
	 * 
	 * Initialize Core subtab and allow to set:
	 * - background color
	 * - background of selection box color
	 * - background selection box opacity
	 * - border of selection box color
	 * - border selection box opacity
	 * - center on focus node activated/unactivated
	 * @method initCore
	 * 
	 */
	initCore : function(){
		var $el = this.$el;
		$el.empty();
		$el.append(
				'<div id="vizstyle_Global" style="">\
					<h2>Background</h2>\
						<div class="property background core" property="core_backgroundColor" type="color">\
							<label class="style">Color <span class="ui-validation-error-message"></span></label>\
							<input  class="" type="text" readonly>\
				       		</div>\
					<h2>Selection Box</h2>\
						<div class="property selectionBox core" property="core_selectionBoxColor" type="color">\
							<label class="style"> Color <span class="ui-validation-error-message"></span></label>\
							<input  class="" type="text"readonly>\
						</div>\
						<div class="property selectionBox core" property="core_selectionBoxBorderColor" type="color">\
							<label class="style"> Border color <span class="ui-validation-error-message"></span></label>\
							<input  class="" type="text" readonly>\
						</div>\
						<div class="property selectionBox core" property="core_selectionBoxBorderWidth" type="number">\
							<label class="style"> Border width <span class="ui-validation-error-message"></span></label>\
							<input  class="" type="text">\
						</div>\
					<h2>Behavior Graph</h2>\
						<div class="property behaviourGraph core" property="core_bubbleBoxCenter" type="focus_checkbox">\
							<label class="style">Center on selected node<span class="ui-validation-error-message"></span></label>\
							<input id=focusNode_Vizstyle type="checkbox" />\
						</div>\
					</div>'
		);
			
		var	style = cy.style();
			
		/* initialize input values */
		$el.find(".property input").each(function(){	
			var $input = $(this);	
			switch( $input.parent().attr("property") ) {
					case "core_backgroundColor":
						$input.val($.Color($("#cy").css('background-color')).toHexString()).css("background-color", $input.val());
						break;
					case "core_selectionBoxColor":
						$input.val(style._private.coreStyle["selection-box-color"].strValue).css("background-color", $input.val());
						break;
					case "core_selectionBoxBorderColor":
						$input.val(style._private.coreStyle["selection-box-border-color"].strValue).css("background-color", $input.val());
						break;
					case "core_selectionBoxBorderWidth":
						$input.val(style._private.coreStyle["selection-box-border-width"].value);
						break;
			}

		}); 

		//add event set color properties
	    this.setColors($el);
	
		//add event set numbers properties
		this.setNumbers($el);
	
		//add event set per cent number properties
		//this.setRates($el);
	
		if(cy._private.centered){
			$(this).prop("checked", true);
		}else{
			$(this).prop("checked", false);
		}
		
		$("#focusNode_Vizstyle").click( function() { 
					if($(this).prop("checked")){
						cy._private.centered = true;
					}else{
						cy._private.centered = false;
					} 
					
		});
	},
	
	/**
	 * 
	 * Initialize nodes subtab and allow to set:
	 * - fill color 
	 * - fill opacity
	 * - fill height
	 * - fill width
	 * - fill shape
	 * - border color
	 * - border weight
	 * - border opacity
	 * - label size
	 * - label weight
	 * - label color
	 * - label vertical position
	 * - label horizontal position
	 * @method initNodes
	 * 
	 */
	//TODO Optimization : copy value of cytoscape.js with loop to write options automaticaly
	initNodes : function () { 
		var $el = this.$el;
		$el.empty(); 
		$el.append(
			   '<div id="vizstyle_Nodes" style="">\
			   		<form>\
							<label class="type" for="allNodes">All Nodes</label>\
							<input type="radio" name="typeof_nodes" id="allNodes" value="allNodes">\
							<label class="type" for="root">Root</label>\
							<input type="radio" name="typeof_nodes" id="root" value="root">\
							<label class="type" for="children">Children</label>\
							<input type="radio" name="typeof_nodes" id="children" value="children">\
					</form>\
					<h2>Border</h2>\
						<div class="property border nodes" property="nodes_borderWidth" type="number">\
							<label class="style"> Width <span class="ui-validation-error-message"></span></label>\
							<input  class="" type="text" ">\
				       		</div>\
						<div class="property border nodes" property="nodes_borderColor" type="color">\
							<label class="style"> Color <span class="ui-validation-error-message"></span></label>\
							<input  class="" type="text" readonly>\
				       		</div>\
					<h2>Fill</h2>\
						<div class="property fill nodes" property="nodes_width" type="number">\
							<label class="style"> Width <span class="ui-validation-error-message"></span></label>\
							<input  class="" type="text">\
						</div>\
						<div class="property fill nodes" property="nodes_height" type="number">\
							<label class="style"> Height <span class="ui-validation-error-message"></span></label>\
							<input  class="" type="text" ">\
						</div>\
						<div class="property fill nodes" property="nodes_backgroundColor" type="color">\
							<label class="style"> Color <span class="ui-validation-error-message"></span></label>\
							<input  class="" type="text" readonly>\
				       		</div>\
						<div class="property fill nodes" property="nodes_shape" type="node_shape">\
							<label class="style"> Shape <span class="ui-validation-error-message"></span></label>\
							<input  class="" type="text" readonly>\
				       		</div>\
					<h2>Label</h2>\
						<div class="property label nodes" property="nodes_labelValign" type="string_selection">\
							<label class="style"> Vertical alignement <span class="ui-validation-error-message"></span></label>\
							<select  class="valign" type="text" >\
								<option value=1 >top</option>\
								<option value=2 >center</option>\
								<option value=3 >bottom</option>\
							</select>\
						</div>\
						<div class="property label nodes" property="nodes_labelHalign" type="string_selection">\
							<label class="style"> Horizontal aligment <span class="ui-validation-error-message"></span></label>\
							<select  class="halign" type="text" ">\
								<option value=1 >right</option>\
								<option value=2 >center</option>\
								<option value=3 >left</option>\
							</select>\
						</div>\
						<div class="property label nodes" property="nodes_labelColor" type="color">\
							<label class="style"> Color <span class="ui-validation-error-message"></span></label>\
							<input  class="" type="text" readonly>\
				       		</div>\
						<div class="property label nodes" property="nodes_labelWeight" type="string_selection">\
							<label class="style"> Weight <span class="ui-validation-error-message"></span></label>\
							<select  class="weight" type="text">\
								<option value=1 >normal</option>\
								<option value=2 >bold</option>\
								<option value=3 >bolder</option>\
								<option value=4 >lighter</option>\
							</select>\
						</div>\
						<div class="property label nodes" property="nodes_labelSize" type="number">\
							<label class="style"> Size <span class="ui-validation-error-message"></span></label>\
							<input  class="" type="text" ">\
				       		</div>\
					</div>'
		);
		
		/* initialize input values */
		var	nodes = cy.nodes();
		$el.find(".property input").each(function(){	
			var $input = $(this);	
			switch( $input.parent().attr("property") ) {
					
					case "nodes_borderColor":
						$input.val(nodes.css("border-color")).css("background-color", $input.val());
						break;
					case "nodes_backgroundColor":
						$input.val(nodes.css("background-color")).css("background-color", $input.val());
						break;
					case "nodes_labelColor":
						$input.val(nodes.css("color")).css("background-color", $input.val());
						break;
					case "nodes_borderWidth":
						$input.val(nodes.css("border-width"));
						break;
					case "nodes_width":
						$input.val(nodes.css("width"));
						break;
					case "nodes_height":
						$input.val(nodes.css("height"));
						break;
					case "nodes_labelSize":
						$input.val(nodes.css("font-size"));
						break;
					case "nodes_shape" : 
		  				$input.val(nodes.css("shape"));
						break;
					case "nodes_labelValign":
						var index = 1;
						$("select[class = valign] > option").each(function (){
							if ( $("select[class = valign] option:selected").html() != nodes.css("text-valign")){
		
								$("select[class = valign] option[selected='selected']").removeAttr("selected"); 
								index ++;
		 						$("select[class = valign] option[value="+index+"]").attr("selected","selected");
							}
								
						});
		           	 	break;
					case "nodes_labelHalign":
						var index = 1;
						$("select[class = halign] > option").each(function (){
							if ( $("select[class = halign] option:selected").html() != nodes.css("text-halign")){
		
								$("select[class = halign] option[selected='selected']").removeAttr("selected"); 
								index ++;
		 						$("select[class = halign] option[value="+index+"]").attr("selected","selected");
							}
			
						});
		            	break;
					case "nodes_labelWeight":
						var index = 1;
						$("select[class = weight] > option").each(function (){
							if ( $("select[class = weight] option:selected").html() != nodes.css("font-weight")){
		
								$("select[class = weight] option[selected='selected']").removeAttr("selected"); 
								index ++;
		 						$("select[class = weight] option[value="+index+"]").attr("selected","selected");
							}
						});
		            	break;
			}
		});
		
		$el.find("#allNodes").prop("checked", "checked");
		
		//change values of vizstyle input with css values of root nodes
		$("#root").click(function(){
		 	//inputUpdate($el);    
		});
		//change values of vizstyle input with css values of root nodes
		$("#allNodes").click(function(){
		 	//inputUpdate($el);    
		});
		$("#children").click(function(){
		 	//inputUpdate($el);  
		});
		
		//add event set color properties
	    this.setColors($el);
	
	    //add event set node shape properties 
	    this.setNodeShape($el);
	
		// add event set numbers properties
		this.setNumbers($el);
	
		// add event set per cent number properties
		//this.setRates($el);
	
		// add event set string selection
		this.setString ($el);
	},
	
	/**
	 * 
	 * Initialize edges subtab and allow to set:
	 * - line color
	 * - line weight
	 * - line opacity
	 * - line type
	 * - line shape
	 * - source color
	 * - source shape
	 * - target color
	 * - target shape
	 * - label size
	 * - label weight
	 * - label color
	 * - label vertical position
	 * - label horizontal position
	 * @method initEdges
	 * 
	 */
	//TODO Optimization : copy value of cytoscape.js with loop to write options automaticaly	
	initEdges : function () {
		var $el = this.$el; 
		$el.empty();  
		$el.append(
			   '<div id="vizstyle_Edges" style="">\
					<h2>Line</h2>\
						<div class="property line edges" property="edges_width" type="number">\
							<label class="style"> Width <span class="ui-validation-error-message"></span></label>\
							<input  class="" type="text" ">\
				       		</div>\
						<div class="property line edges" property="edges_lineColor" type="color">\
							<label class="style"> Color <span class="ui-validation-error-message"></span></label>\
							<input  class="" type="text" readonly>\
				       		</div>\
						<div class="property line edges" property="edges_curve" type="string_selection">\
							<label class="style"> curve <span class="ui-validation-error-message"></span></label>\
							<select  class="curve" type="text">\
								<option value=1>straight</option>\
								<option value=2>bezier</option>\
								<option value=3>bundled</option>\
							</select>\
				       		</div>\
						<div class="property line edges" property="edges_controlPointStepSize" type="number">\
							<label class="style"> Control point step size <span class="ui-validation-error-message"></span></label>\
							<input  class="" type="text" ">\
				       		</div>\
						<div class="property line edges" property="edges_lineStyle" type="line_shape">\
							<label class="style"> Shape <span class="ui-validation-error-message"></span></label>\
							<input  class="" type="text" readonly>\
				       		</div>\
					<h2>Target</h2>\
						<div class="property target edges" property="edges_targetArrowShape" type="edge_shape">\
							<label class="style"> Target Shape <span class="ui-validation-error-message"></span></label>\
							<input  class="" type="text" readonly>\
						</div>\
						<div class="property target edges" property="edges_targetArrowColor" type="color">\
							<label class="style"> Color <span class="ui-validation-error-message"></span></label>\
							<input  class="" type="text" readonly>\
				       		</div>\
					<h2>Source</h2>\
						<div class="property source edges" property="edges_sourceArrowShape" type="edge_shape">\
							<label class="style"> Source Shape <span class="ui-validation-error-message"></span></label>\
							<input  class="" type="text" readonly>\
						</div>\
						<div class="property source edges" property="edges_sourceArrowColor" type="color">\
							<label class="style"> Color <span class="ui-validation-error-message"></span></label>\
							<input  class="" type="text" readonly>\
				       		</div>\
					<h2>Label</h2>\
						<div class="property label edges" property="edges_labelValign" type="string_selection">\
							<label class="style"> Vertical alignement <span class="ui-validation-error-message"></span></label>\
							<select  class="valign" type="text">\
								<option value=1 >top</option>\
								<option value=2 >center</option>\
								<option value=3 >bottom</option>\
							</select>\
						</div>\
						<div class="property label edges" property="edges_labelHalign" type="string_selection">\
							<label class="style"> Horizontal aligment <span class="ui-validation-error-message"></span></label>\
							<select  class="halign" type="text" ">\
								<option value=1 >right</option>\
								<option value=2 >center</option>\
								<option value=3 >left</option>\
							</select>\
						</div>\
						<div class="property label edges" property="edges_labelColor" type="color">\
							<label class="style"> Color <span class="ui-validation-error-message"></span></label>\
							<input  class="" type="text" readonly>\
				       		</div>\
						<div class="property label edges" property="edges_labelWeight" type="string_selection">\
							<label class="style"> Weight <span class="ui-validation-error-message"></span></label>\
							<select  class="weight" type="text" ">\
								<option value=1 >normal</option>\
								<option value=2 >bold</option>\
								<option value=3 >bolder</option>\
								<option value=4 >lighter</option>\
							</select>\
						</div>\
						<div class="property label edges" property="edges_labelSize" type="number">\
							<label class="style"> Size <span class="ui-validation-error-message"></span></label>\
							<input  class="" type="text" ">\
				       		</div>\
					</div>'
			);
			
			var edges = cy.edges();
			$el.find(".property input").each(function(){	
				var $input = $(this);
				switch( $input.parent().attr("property") ) {
	           		case "edges_lineColor":
						$input.val(edges.css("line-color")).css("background-color", $input.val());
						break;
					case "edges_lineStyle":
						$input.val(edges.css("line-style")).css("line-style" , $input.val());
						break;
					case "edges_targetArrowColor":
						$input.val(edges.css("target-arrow-color")).css("background-color", $input.val());
						break;
					case "edges_sourceArrowColor":
						$input.val(edges.css("source-arrow-color")).css("background-color", $input.val());
						break;
					case "edges_labelColor":
						$input.val(edges.css("color")).css("background-color", $input.val());
						break;
					case "edges_width":
						$input.val(edges.css("width"));
						break;
					case "edges_controlPointStepSize":
						$input.val(edges.css("control-point-step-size"));
						break;
					case "edges_labelSize":
						$input.val(edges.css("font-size"));
						break;	
					case "edges_targetArrowShape":
						$input.val(edges.css("target-arrow-shape")).css("target-arrow-shape", $input.val());
						break;
					case "edges_sourceArrowShape":
						$input.val(edges.css("source-arrow-shape")).css("source-arrow-shape", $input.val());
						break;
					case "edges_labelValign":
						var index = 1;
						$("select[class = valign] > option").each(function (){
							if ( $("select[class = valign] option:selected").html() != edges.css("text-valign")){
		
								$("select[class = valign] option[selected='selected']").removeAttr("selected"); 
								index ++;
		 						$("select[class = valign] option[value="+index+"]").attr("selected","selected");
							}
								
						});
		            	break;
					case "edges_labelHalign":
						var index = 1;
						$("select[class = halign] > option").each(function (){
							if ( $("select[class = halign] option:selected").html() != edges.css("text-halign")){
		
								$("select[class = halign] option[selected='selected']").removeAttr("selected"); 
								index ++;
		 						$("select[class = halign] option[value="+index+"]").attr("selected","selected");
							}
			
						});
		            	break;
					case "edges_labelWeight":
						var index = 1;
						$("select[class = weight] > option").each(function (){
							if ( $("select[class = weight] option:selected").html() != edges.css("font-weight")){
		
								$("select[class = weight] option[selected='selected']").removeAttr("selected"); 
								index ++;
		 						$("select[class = weight] option[value="+index+"]").attr("selected","selected");
							}
						});
		            	break;
						
				}
		 });	
			
			//set color properties
		    this.setColors($el);
		
		    // set edge shape properties
		    this.setEdgeShape($el);
			//set line shape properties
			this.setLineShape($el);
		
			//set numbers properties
			this.setNumbers($el);
		
			//set per cent number properties
			//this.setRates($el);
		
			//set string selection
			this.setString($el);
		},
		  

		/**
		 * Initialize color input values and display color picker when focusin on color input
		 * @event setColors
		 * @param {Object} $el jQuery selector for input
		 */ 
		 setColors : function ($el) {
			var nodes, 
				edges = cy.edges(), 
				style = cy.style();

			//initialize variable nodes to decide witch nodes can change his style
			if ($("#vizstyle_Nodes > form").find("input:checked").val() == "root") {
				nodes = cy._private.root;
			} else {
				nodes = cy.nodes();
			}

			$el.find(".property[type=color] input").each(function() {

				var $input = $(this);
				$input.addClass("color-chooser");

				/*add picker near input when clicked */
				$input.focusin(function() {
					if (!$.layout.browser.msie) {
						$(".colorpicker .header").hide();
					}

					/*initialize variable nodes to decide witch nodes can change his style*/
					if ($("#vizstyle_Nodes > form").find("input:checked").val() == "root") {
						nodes = cy._private.root;
					} else {
						nodes = cy.nodes();
					}

					switch( $input.parent().attr("property") ) {
						case "nodes_backgroundOpacity":
							nodes.css("background-opacity", $input.val()).data("previousCss").backgroundOpacity = $input.val();
							break;
						case "edges_opacity":
							edges.css("opacity", $input.val()).data("previousCss").opacity = $input.val();
							break;
					};




					
					var pickerInternals = $input.picker({

						color: $input.val(),
						onShow : function(colpkr) {

							//cy.off("mouseout mouseover touchout touchover");
						},
						onHide : function(colpkr) {
							cy.elements(":selected").unselect();
							switch( $input.parent().attr("property") ) {
								case "nodes_backgroundColor":
									nodes.css("background-color", $input.val()).data("previousCss").backgroundColor = $input.val();
									break;
								case "nodes_borderColor":
									nodes.css("border-color", $input.val()).data("previousCss").borderColor = $input.val();
									break;
								case "nodes_labelColor":
									nodes.css("color", $input.val()).data("previousCss").labelColor = $input.val();
									break;
								case "edges_lineColor":
									edges.css("line-color", $input.val()).data("previousCss").lineColor = $input.val();
									break;
								case "edges_targetArrowColor":
									edges.css("target-arrow-color", $input.val()).data("previousCss").targetArrowColor = $input.val();
									break;
								case "edges_sourceArrowColor":
									edges.css("source-arrow-color", $input.val()).data("previousCss").sourceArrowColor = $input.val();
									break;
								case "edges_labelColor":
									edges.css("color", $input.val()).data("previousCss").labelColor = $input.val();
									break;

							};
							
							//TODO:opacity
							
							/*apply style to all nodes without root*/
							var root = cy._private.root;

							if ($("#vizstyle_Nodes > form").find("input:checked").val() == "children") {
								var previousCss = root.data("previousCss");
								root.css({
									backgroundOpacity : previousCss.backgroundOpacity,
									borderOpacity : previousCss.borderOpacity,
									opacity : previousCss.opacity,
									overlayOpacity : previousCss.overlayOpacity,
									textOutlineOpacity : previousCss.textOutlineOpacity,
									backgroundColor : previousCss.backgroundColor,
									backgroundImage : previousCss.backgroundImage,
									borderColor : previousCss.borderColor,
									color : previousCss.color,
									lineColor : previousCss.lineColor,
									lineStyle : previousCss.lineStyle,
									overlayColor : previousCss.overlayColor,
									targetArrowColor : previousCss.targetArrowColor,
									textOutlineColor : previousCss.textOutlineColor,
								});
							} else {
								root.data("previousCss", root.css());
							};

							return;
						},
						onSubmit : function() {
							cy.elements(":selected").unselect();
							switch( $input.parent().attr("property") ) {
								case "nodes_backgroundColor":
									nodes.css("background-color", $input.val()).data("previousCss").backgroundColor = $input.val();
									break;
								case "nodes_borderColor":
									nodes.css("border-color", $input.val()).data("previousCss").borderColor = $input.val();
									break;
								case "nodes_labelColor":
									nodes.css("color", $input.val()).data("previousCss").labelColor = $input.val();
									break;
								case "edges_lineColor":
									edges.css("line-color", $input.val()).data("previousCss").lineColor = $input.val();
									break;
								case "edges_targetArrowColor":
									edges.css("target-arrow-color", $input.val()).data("previousCss").targetArrowColor = $input.val();
									break;
								case "edges_sourceArrowColor":
									edges.css("source-arrow-color", $input.val()).data("previousCss").sourceArrowColor = $input.val();
									break;
								case "edges_labelColor":
									edges.css("color", $input.val()).data("previousCss").labelColor = $input.val();
									break;

									//TODO:opacity
									
									/*apply style to all nodes without root*/
									var root = cy._private.root;
									if ($("#vizstyle_Nodes > form").find("input:checked").val() == "children") {

										var previousCss = root.data("previousCss");

										root.css({
											backgroundOpacity : previousCss.backgroundOpacity,
											borderOpacity : previousCss.borderOpacity,
											opacity : previousCss.opacity,
											overlayOpacity : previousCss.overlayOpacity,
											backgroundColor : previousCss.backgroundColor,
											backgroundImage : previousCss.backgroundImage,
											borderColor : previousCss.borderColor,
											color : previousCss.color,
											lineColor : previousCss.lineColor,
											lineStyle : previousCss.lineStyle,
											overlayColor : previousCss.overlayColor,
											targetArrowColor : previousCss.targetArrowColor,
											textOutlineColor : previousCss.textOutlineColor,
										});
									} else {
										root.data("previousCss", root.css());
									};

							};
						},
						onChangeColor : function(col) {
							var colorSelected = "#"+$.picker.hsbToHex(col);
							switch( $input.parent().attr("property") ) {
								case "core_backgroundColor":
									$("#cy").css("background-color", colorSelected);
									$input.val(colorSelected).css("background-color", colorSelected);
									break;
								case "core_selectionBoxColor":
									style.selector("core").css("selection-box-color", colorSelected);
									$input.val(colorSelected).css("background-color", colorSelected);
									break;
								case "core_selectionBoxBorderColor":
									style.selector("core").css("selection-box-border-color", colorSelected);
									$input.val(colorSelected).css("background-color", colorSelected);
									break;
								case "nodes_backgroundColor":
									$input.val(colorSelected).css("background-color", colorSelected);
									break;
								case "nodes_borderColor":
									$input.val(colorSelected).css("background-color", colorSelected);
									break;
								case "nodes_labelColor":
									$input.val(colorSelected).css("background-color", colorSelected);
									break;
								case "edges_lineColor":
									$input.val(colorSelected).css("background-color", colorSelected);
									break;
								case "edges_targetArrowColor":
									$input.val(colorSelected).css("background-color", colorSelected);
									break;
								case "edges_sourceArrowColor":
									$input.val(colorSelected).css("background-color", colorSelected);
									break;
								case "edges_labelColor":
									$input.val(colorSelected).css("background-color", colorSelected);
									break;

							};
							
							//TODO:opacity

						}
					});

				});

			});
		},

	
		/**
		 * Initialize number input values and define action after foscusout
		 * @event setNumbers
		 * @param {Object} $el jQuery selector for input
		 */
		setNumbers : function ($el) {
			var nodes, 
				edges = cy.edges(), 
				style = cy.style();

			//initialize variable nodes to decide witch nodes can change his style
			if ($("#vizstyle_Nodes > form").find("input:checked").val() == "root") {
				nodes = cy._private.root;
			} else {
				nodes = cy.nodes();
			}

			$el.find(".property[type = number] input").each(function() {
				var $input = $(this);

				//initialize input values
				//inputNumberInit($input, nodes, edges, style);

				/*set input values*/
				$input.on("focusout", function() {

					/*initialize variable nodes to decide witch nodes can change his style*/
					if ($("#vizstyle_Nodes > form").find("input:checked").val() == "root") {
						nodes = cy._private.root;
					} else {
						nodes = cy.nodes();
					}

					switch( $input.parent().attr("property") ) {
						case "core_selectionBoxBorderWidth":
							style.selector("core").css("selection-box-border-width", $input.val());
							break;
						case "nodes_borderWidth":
							nodes.css("border-width", $input.val()).data("previousCss").borderWidth = $input.val();
							break;
						case "nodes_width":
							nodes.css("width", $input.val()).data("previousCss").width = $input.val();
							break;
						case "nodes_height":
							nodes.css("height", $input.val()).data("previousCss").height = $input.val();
							break;
						case "nodes_labelSize":
							nodes.css("font-size", $input.val()).data("previousCss").fontSize = $input.val();
							break;
						case "edges_width":
							edges.css("width", $input.val()).data("previousCss").width = $input.val();
							break;
						case "edges_controlPointStepSize":
							edges.css("control-point-step-size", $input.val()).data("previousCss").controlPointStepSize = $input.val();
							
							nodes.each(function() {//fix no update view
								var positionX = this.position("x");
								this.position("x", positionX + 1);
								this.position("x", positionX - 1);
							});
							break;
						case "edges_labelSize":
							edges.css("font-size", $input.val()).data("previousCss").fontSize = $input.val();
							break;
					};

					/*apply style to all nodes without root*/
					var root = cy._private.root;
					if ($("#vizstyle_Nodes > form").find("input:checked").val() == "children") {

						var previousCss = root.data("previousCss");

						root.css({
							borderWidth : previousCss.borderWidth,
							fontSize : previousCss.fontSize,
							height : previousCss.height,
							textOutlineWidth : previousCss.textOutlineWidth,
							width : previousCss.width,
						});

					} else {
						root.data("previousCss", root.css());
					};

				}).keypress(function(e) {
					if (e.which == 13) {
						$input.trigger("focusout");
					}
				});
			});

		},
		

		/**
		 * Initialize rate input values and define action after foscusout
		 * This method is not activated
		 * @event setRates
		 * @depreciate
		 * @param {Object} $el jQuery selector for input
		 */ 
		 /*setRates : function ($el) {
			var nodes, edges = cy.edges(), style = cy.style();

			//initialize variable nodes to decide witch nodes can change his style
			if ($("#vizstyle_Nodes > form").find("input:checked").val() == "root") {
				nodes = cy._private.root;
			} else {
				nodes = cy.nodes();
			}

			$el.find(".property[type =per_cent_number] input").each(function() {
				var $input = $(this);
				//// initialize input values////
				//inputRateInit($input, nodes, edges, style);

				//set input values 
				$input.on("focusout", function() {

					//initialize variable nodes to decide witch nodes can change his style
					if ($("#vizstyle_Nodes > form").find("input:checked").val() == "root") {
						nodes = cy._private.root;
					} else {
						nodes = cy.nodes();
					}

					switch( $input.parent().attr("property") ) {
					};

					//apply style to all nodes without root
					var root = cy._private.root;
					if ($("#vizstyle_Nodes > form").find("input:checked").val() == "children") {

						var previousCss = root.data("previousCss");

						root.css({
						});

					} else {
						root.data("previousCss", root.css());
					};

				}).keypress(function(e) {
					if (e.which == 13) {
						$input.trigger("focusout");
					}
				});
			});
		},*/
		
		/**
		 * Initialize nodes shape input values and define shape picker when focusin on nodes shape input
		 * @event setNodeShape
		 * @param {Object} $el jQuery selector for input
		 */
		setNodeShape : function ($el) {
	
			var self = this;
			var nodes;
	
				/*initialize variable nodes to decide witch nodes can change his style*/
				if($("#vizstyle_Nodes > form").find("input:checked").val() == "root"){
							nodes=cy._private.root;			
				}else{
							nodes=cy.nodes();
				}
			
		       	//inputShapeInit($input, nodes);
		        var $input = $el.find(".property[type=node_shape] input");
				$input
		        	.addClass("shape-chooser nodes_shapes")
		           	.css("shape", $input.val())
		            /*add picker near input when clicked */
		         	.on("focusin", function(){
		         		
		         		/*initialize variable nodes to decide witch nodes can change his style*/
						if($("#vizstyle_Nodes > form").find("input:checked").val() == "root"){
									nodes=cy._private.root;			
						}else{
									nodes=cy.nodes();
						}
		            
		          	var pickerInternals = $(this).picker({
		                	
							shape: "ellipse",
							onShow: function (shapkr) {
									//cy.off("mouseout mouseover touchout touchover");
							},
							onHide: function () {
							},
							onChangeShape: function () {						
											nodes.data("previousCss").shape = $input.val();
											nodes.css("shape", this.data("picker").shape);
											$input.val(this.data("picker").shape);
											if($("#vizstyle_Nodes > form").find("input:checked").val() == "children"){
												var root=cy._private.root;					
												var previousCss = root.data("previousCss");
													root.css("shape", previousCss.shape);
											}
											
											// force current shape when root is selected
											var rootId = nodes.data("id");
											cy.style().selector('#'+rootId+':selected').css("shape", $input.val());
							},
						
		
		
						});
    
					})
		        .keypress(function(e) {
				    if(e.which == 13) {
				       //TODO: same event at onHide
			    	}
			    });
			    
			    
		        
		},
			
		/**
		 * Initialize line shape input values and define shape picker when focusin on line shape input	
		 * @event setLineShape
		 * @param {Object} $el jQuery selector for input
		 */ 
		 setLineShape : function($el) {
			var edges = cy.edges();

			//$el.find(".property[type=line_shape] input").each(function() {
				var $input = $el.find(".property[type=line_shape] input"); //$(this)

				/*add picker near input when clicked*/
				$input
					.addClass("shape-chooser edges_lines")
					.on("focusin", function() {
						$(this).picker({
							shape: "solid",
							onShow: function (shapkr) {
									//cy.off("mouseout mouseover touchout touchover");
							},
							onHide: function () {
							},
							onChangeShape: function () {						
											edges.data("previousCss").shape = $input.val();
											edges.css("line-style", this.data("picker").shape);
											$input.val(this.data("picker").shape);
							},
						
		
		
						});
				}).keypress(function(e) {
					if (e.which == 13) {
						//TODO:
					}
				});
			//});
		},
		
	
		/**
		 * Initialize edges shape input (target shape, source shape) values and define action after focusout
		 * @event setEdgeShape
		 * @param {Object} $el jQuery selector for input
		 */ 
		setEdgeShape: function ($el){
			   var edges = cy.edges();
			   var arrowShape;
		           	
			   $el.find(".property[type=edge_shape] input").each(function(){
		       
		       var $input = $(this);
		           
		       
		       if($input.parent().attr("property") == "edges_targetArrowShape"){
						$input.addClass("shape-chooser edges_target");
				}else{
						$input.addClass("shape-chooser edges_source");
				}
		       $input
		            //// add picker near input when clicked ////
		         	.on("focusin", function(){
		            	
		               var pickerInternals = $(this).picker({
							onShow: function (shapkr) {
									//cy.off("mouseout mouseover touchout touchover");
							},
							onHide: function () {	
							},
							onChangeShape: function () {						
								switch( $input.parent().attr("property")) {
									case "edges_targetArrowShape":
										edges.data("previousCss").targetArrowShape = $input.val();
										edges.css("target-arrow-shape", this.data("picker").shape);
										$input.val(this.data("picker").shape);
										break;
									case "edges_sourceArrowShape":
										edges.data("previousCss").sourceArrowShape = $input.val();
										edges.css("source-arrow-shape", this.data("picker").shape);
										$input.val(this.data("picker").shape);
										break;
								}
							},
						
		
		
						});
						
					if (pickerInternals != undefined) {
						pickerInternals.pickerSetValues(undefined, undefined, undefined, undefined, undefined, $input.val());
					}	             
		                
		            })
			   .keypress(function(e) {
			    if(e.which == 13) {
			       //TODO
		    	}
		  });
		
		  });
		
		},
			
		/**
		 * Initialize string input values and define shape picker when focusin on edge shape input
		 * @event setEdgeShape
		 * @param {Object} $el jQuery selector for input
		 */ 
		setString : function ($el) {
			var nodes, 
				edges = cy.edges();
	
			/*initialize variable nodes to decide witch nodes can change his style*/
			if ($("#vizstyle_Nodes > form").find("input:checked").val() == "root") {
				nodes = cy._private.root;
			} else {
				nodes = cy.nodes();
			}
	
			$el.find(".property[type = string_selection] select").each(function() {
				var $select = $(this);
	
				/*set input values */
				$select.click(function() {
	
					//initialize variable nodes to decide witch nodes can change his style
					if ($("#vizstyle_Nodes > form").find("input:checked").val() == "root") {
						nodes = cy._private.root;
					} else {
						nodes = cy.nodes();
					}
	
					switch( $select.parent().attr("property") ) {
						case "nodes_labelValign":
							if ($("select[class = valign] option:selected").html() != nodes.css("text-valign")) {
								$("select[class = valign] option[selected='selected']").removeAttr("selected");
								$("select[class = valign] > option:selected").attr("selected", "selected");
							};
							nodes.css("text-valign", $("select[class = valign] > option:selected").text()).data("previousCss").textValign = $("select[class = valign] > option:selected").text();
							break;
						case "nodes_labelHalign":
							if ($("select[class = halign] option:selected").html() != nodes.css("text-halign")) {
								$("select[class = halign] option[selected='selected']").removeAttr("selected");
								$("select[class = halign] > option:selected").attr("selected", "selected");
							};
							nodes.css("text-halign", $("select[class = halign] > option:selected").text()).data("previousCss").textHalign = $("select[class = halign] > option:selected").text();
	
							break;
						case "nodes_labelWeight":
							if ($("select[class = weight] option:selected").html() != nodes.css("font-weight")) {
								$("select[class = weight] option[selected='selected']").removeAttr("selected");
								$("select[class = weight] > option:selected").prop("selected", "selected");
							};
							nodes.css("font-weight", $("select[class = weight] > option:selected").text()).data("previousCss").fontWeight = $("select[class = weight] > option:selected").text();
	
							break;
						case "edges_curve":
							if ($("select[class = curve] > option:selected").html() != edges.css("curve-style")) {
								$("select[class = curve] option[selected='selected']").removeAttr("selected");
								$("select[class = curve] > option:selected").prop("selected", "selected");
							};
	
							//implement fake straight edges style
							if ($("select[class = curve] > option:selected").html() == "straight") {
								edges.css({
									curveStyle : "bezier",
									controlPointStepSize : 1
								});
								edges.data("previousCss").curveStyle = "bezier";
								edges.data("previousCss").controlPointStepSize = 1;
								$select.parent().siblings().filter("[ property = edges_controlPointStepSize]").find("input").prop('disabled', true);
	
							} else {
								edges.css({
									curveStyle : $("select[class = curve] > option:selected").html(),
									controlPointStepSize : 40
								});
								edges.data("previousCss").curveStyle = "bezier";
								edges.data("previousCss").controlPointStepSize = 40;
								$select.parent().siblings().filter("[ property = edges_controlPointStepSize]").find("input").val("40px").prop('disabled', false);
	
							}
							//fix no update view
							nodes.each(function() {
								var positionX = this.position("x");
								this.position("x", positionX + 1);
								this.position("x", positionX - 1);
							});
	
							break;
						case "edges_labelValign":
							if ($("select[class = valign] option:selected").html() != edges.css("text-valign")) {
								$("select[class = valign] option[selected='selected']").removeAttr("selected");
								$("select[class = valign] > option:selected").attr("selected", "selected");
							};
							edges.css("text-valign", $("select[class = valign] > option:selected").text()).data("previousCss").textValign = $("select[class = valign] > option:selected").text();
							break;
						case "edges_labelHalign":
							if ($("select[class = halign] option:selected").html() != edges.css("text-halign")) {
								$("select[class = halign] option[selected='selected']").removeAttr("selected");
								$("select[class = halign] > option:selected").attr("selected", "selected");
							};
							edges.css("text-halign", $("select[class = halign] > option:selected").text()).data("previousCss").textHalign = $("select[class = halign] > option:selected").text();
							break;
						case "edges_labelWeight":
							if ($("select[class = weight] option:selected").html() != edges.css("font-weight")) {
								$("select[class = weight] option[selected='selected']").removeAttr("selected");
								$("select[class = weight] > option:selected").attr("selected", "selected");
							};
							edges.css("font-weight", $("select[class = weight] > option:selected").text()).data("previousCss").fontWeight = $("select[class = weight] > option:selected").text();
							break;
					};
	
					//apply style to all nodes without root
					var root = cy._private.root;
					if ($("#vizstyle_Nodes > form").find("input:checked").val() == "children") {
	
						var previousCss = root.data("previousCss");
	
						root.css({
							fontWeight : previousCss.fontWeight,
							textHalign : previousCss.textHalign,
							textValign : previousCss.textValign,
						});
	
					} else {
						root.data("previousCss", root.css());
					};
				});
	
			});
		},

	};
	
/*	function inputUpdate($el){
		var	nodes,
			edges = cy.edges(),
			style = cy.style();
		//initialize variable nodes to decide witch nodes can change his style
		if($("#vizstyle_Nodes > form").find("input:checked").val() == "root"){
					nodes=cy._private.root;			
		}else{
					nodes=cy.nodes();
		}
				
		 $el.find(".property input").each(function(){
		 	
	        var $input = $(this);
		    //// initialize input values////
		     //initColors($input);
		     //inputShapeInit($input);
		     //inputNumberInit($input);
		     //inputRateInit ($input);
   
	      });
	       
	      $el.find(".property select").each(function(){
	       	  
	       	  var $select = $(this);
	       	 //// initialize select values////
	       	 inputStringInit($select, nodes, edges);
	      });
	}*/

	var methods={
		/**
		 * Initialize and display vizstyle tab.
		 * @method init
 		 * @param {Object} startStyle the type of style editor (core,node, edges).
 		 * @for vizStyle
		 */
		init : function(startStyle){
			var $el= $(this);	
			var vizStyle = new VizStyle($el);
			$el.data("VizStyle", vizStyle);
			switch(startStyle) {
                case "core":
					vizStyle.initCore($el);
				break;
				case "node":
					vizStyle.initNodes($el);
				break;
				case "edge":
					vizStyle.initEdges($el);
				break;
			};
		return vizStyle;
		}
	};
	
	/** extend jquery functions
	 * 
	 * @namespace
	 */
	$.fn.extend({
			vizStyle: methods.init
	});

})(jQuery);