/**
 * @summary    jcanvas.lite
 * @description Draw elements in canvas with jquery 
 * @version     
 * @file        jcanvas.lite.js
 * @author      Alexan Andrieux
 * @contact     <a href="mailto:alexan.andrieux@inria.fr"> Alexan Andrieux</a>
 * @category
 *
 * @copyright 
 *
 * This source file is free software, under AGPL License.
 * 
 * This source file is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the license files for details.
 *
 * For details please refer to: <a href="http://www.gnu.org/licenses/agpl-3.0.html"></a>
 */

/**
 * @license jCanvas
 * Based on Caleb Evans jCanvas <a href="http://calebevans.me/projects/jcanvas/"</a> licensed under MIT license
 * 
 */

(function($, Math, parseFloat, TRUE, FALSE, NULL, UNDEFINED) {

/* Define local aliases to frequently used properties */
var defaults,
	extendObject = $.extend,
	inArray = $.inArray,
	maps = {},
	typeOf = $.type,
	isFunction = $.isFunction,
	isPlainObject = $.isPlainObject,
	round = Math.round,
	PI = Math.PI,
	sin = Math.sin,
	cos = Math.cos,
	atan2 = Math.atan2,
	drawingMap;

/**
 *  Preferences constructor (which inherits from the defaults object)
 * 
 * @class jCanvasObject
 * @constructor
 * @param {array} args the defauts parameters
 */
function jCanvasObject(args) {
	var params = this;
	// Merge preferences with arguments
	extendObject(params, args);
	return params;
}

 
/**
 *  jCanvas function for setting property defaults (it's also an object)
 * 
 * @class jCanvas
 * @param {array} args the defauts parameters
 */
function jCanvas(args) {
	if (args) {
		// Merge arguments with preferences
		extendObject(jCanvasObject.prototype, args);
	} else {
		// Reset preferences to defaults if nothing is passed
		jCanvas.prefs = jCanvasObject.prototype = extendObject({}, defaults);
	}
	return this;
}
// Allow jCanvas function to be "chained" to other methods
$.fn.jCanvas = jCanvas;

// jCanvas default property values
defaults = {
	align: 'center',
	arrowAngle: 90,
	arrowRadius: 0,
	autosave: TRUE,
	baseline: 'middle',
	bringToFront: FALSE,
	replaceLine: FALSE,
	ccw: FALSE,
	closed: FALSE,
	compositing: 'source-over',
	concavity: 0,
	cornerRadius: 0,
	count: 1,
	cropFromCenter: TRUE,
	dragGroups: NULL,
	group: NULL,
	groups: NULL,
	data: {},
	dx: NULL,
	dy: NULL,
	each: NULL,
	end: 360,
	fillStyle: 'transparent',
	fontStyle: 'normal',
	fontSize: '12pt',
	fontFamily: 'sans-serif',
	fromCenter: TRUE,
	fn: NULL,
	height: NULL,
	imageSmoothing: TRUE,
	inDegrees: TRUE,
	index: NULL,
	lineHeight: 1,
	layer: FALSE,
	maxWidth: NULL,
	miterLimit: 10,
	name: NULL,
	opacity: 1,
	r1: NULL,
	r2: NULL,
	radius: 0,
	repeat: 'repeat',
	respectAlign: FALSE,
	rounded: FALSE,
	rotate:0,
	shadowBlur: 0,
	shadowColor: 'transparent',
	shadowStroke: false,
	shadowX: 0,
	shadowY: 0,
	sHeight: NULL,
	sides: 0,
	source: '',
	spread: 0,
	start: 0,
	strokeCap: 'butt',
	strokeDash: NULL,
	strokeDashOffset: 0,
	strokeJoin: 'miter',
	strokeStyle: 'transparent',
	strokeWidth: 1,
	sWidth: NULL,
	sx: NULL,
	sy: NULL,
	width: NULL,
	x: 0,
	y: 0
};

// Copy defaults to preferences object
jCanvas();

/* Internal helper methods */

/**
 * Determines if the given operand is a string.
 * @method isString
 * @param {string/int/object/array} operandseq the variable to test.
 * @private
 */
function isString(operand) {
	return (typeOf(operand) === 'string');
}


/**
 * Get 2D context for the given canvas
 * @method _getContext
 * @param {object} canvas the canvas selected.
 * @private 
 */ 
function _getContext(canvas) {
	return (canvas && canvas.getContext ? canvas.getContext('2d') : NULL);
}


/**
 * Set the style with the given name.
 * @method _setStyle
 * @param {object} canvas the canvas selected.
 * @param {object} ctx the 2D context of the canvas.
 * @param {object} params 
 * @param {string} styleName the name of the style
 * @private
 */
function _setStyle( canvas, ctx, params, styleName ) {
	if ( params[ styleName ] ) {
		if ( isFunction( params[ styleName ] ) ) {
			// Handle functions
			ctx[ styleName ] = params[ styleName ].call( canvas, params );
		} else {
			// Handle string values
			ctx[ styleName ] = params[ styleName ];
		}
	}
}

 
/**
 * Set canvas context properties.
 * @method _setGlobalProps
 * @param {object} canvas the canvas selected.
 * @param {object} ctx the 2D context of the canvas.
 * @param {object} params 
 * @private
 */
function _setGlobalProps( canvas, ctx, params ) {
	_setStyle( canvas, ctx, params, 'fillStyle' );
	_setStyle( canvas, ctx, params, 'strokeStyle' );

	ctx.lineWidth = params.strokeWidth;
	// Optionally round corners for paths
	if ( params.rounded ) {
		ctx.lineCap = ctx.lineJoin = 'round';
	} else {
		ctx.lineCap = params.strokeCap;
		ctx.lineJoin = params.strokeJoin;
		ctx.miterLimit = params.miterLimit;
	}
	// Optionally dash lines
	if ( params.strokeDash ) {
		if (typeof ctx.setLineDash == "function"){
			ctx.setLineDash( params.strokeDash );
			ctx.webkitLineDash = ctx.mozDash = params.strokeDash;
			ctx.lineDashOffset = ctx.webkitLineDashOffset = ctx.mozDashOffset = params.strokeDashOffset;
			params.replaceLine = FALSE;
		
		}else{
			drawDashedLine(canvas, ctx, params);
			params.replaceLine = TRUE;
		}
	}else{
		params.replaceLine = FALSE;
	}
	
	// Opacity operation
	ctx.globalAlpha = params.opacity;
	
}

/* Layer API */

/**
 * Retrieved the stored jCanvas data for a canvas element.
 * @method _getCanvasData
 * @param {object} canvas the canvas selected.
 * @private
 */
function _getCanvasData(canvas) {
	var data;
	// Get canvas data
	data = $.data(canvas, 'jCanvas');
	if (!data) {
		// Create canvas data object if it does not already exist
		data = {
			// The associated canvas element
			canvas: canvas,
			// Layers array
			layers: [],
			// Layer maps
			layer: {
				names: {},
				groups: {}
			}
		};
		// Use jQuery to store canvas data
		$.data(canvas, 'jCanvas', data);
			
	}
		
	return data;
}


/**
 * Update a layer property map if property is changed
 * @method _updateLayerName
 * @param {object} data the data.
 * @param {object} layer the layer selected.
 * @param {object} props the new layer
 * @private
 */
function _updateLayerName(data, layer, props) {
	var nameMap = data.layer.names;
	
	// If layer name is being added, not changed
	if (!props) {
		
		props = layer;
		
	} else {
		
		// Remove old layer name entry because layer name has changed
		if (props.name !== UNDEFINED && isString(layer.name) && layer.name !== props.name) {
			delete nameMap[layer.name];
		}
		
	}
	
	// Add new entry to layer name map with new name
	if (isString(props.name)) {
		nameMap[props.name] = layer;
	}
}

/**
 * Create or update the data map for the given layer and group type
 * @method _updateLayerGroups
 * @param {object} data the data.
 * @param {object} layer the layer selected.
 * @param {object} props the new layer
 * @private
 */
function _updateLayerGroups(data, layer, props) {
	var groupMap = data.layer.groups,
		group, groupName, g,
		index, l;
	
	// Fallback for deprecated group property
	if (layer.group !== NULL) {
		layer.groups = [layer.group];
		if (layer.dragGroupWithLayer) {
			layer.dragGroups = [layer.group];
		}
	}
	if (props && props.group !== UNDEFINED) {
		if (props.group === NULL) {
			props.groups = NULL;
		} else {
			props.groups = [props.group];
			if (props.dragGroupWithLayer) {
				props.dragGroups = [props.group];
			}
		}
	}
	
	// If group name is not changing
	if (!props) {
		
		props = layer;
		
	} else {
		
		// Remove layer from all of its associated groups
		if (props.groups !== UNDEFINED && layer.groups !== NULL) {
			for (g = 0; g < layer.groups.length; g += 1) {
				groupName = layer.groups[g];
				group = groupMap[groupName];
				if (group) {
					// Remove layer from its old layer group entry
					for (l = 0; l < group.length; l += 1) {
						if (group[l] === layer) {
							// Keep track of the layer's initial index
							index = l;
							// Remove layer once found
							group.splice(l, 1);
							break;
						}
					}
					// Remove layer group entry if group is empty
					if (group.length === 0) {
						delete groupMap[groupName];
					}
				}
			}
		}
		
	}
		
	// Add layer to new group if a new group name is given
	if (props.groups !== UNDEFINED && props.groups !== NULL) {
				
		for (g = 0; g < props.groups.length; g += 1) {
			
			groupName = props.groups[g];
			
			group = groupMap[groupName];
			if (!group) {
				// Create new group entry if it doesn't exist
				group = groupMap[groupName] = [];
				group.name = groupName;
			}
			if (index === UNDEFINED) {
				// Add layer to end of group unless otherwise stated
				index = group.length;
			}
			// Add layer to its new layer group
			group.splice(index, 0, layer);
		
		}
		
	}
}
 
/**
 * Get jCanvas layers array
 * @method getLayers
 */
$.fn.getLayers = function getLayers() {
	var canvas = this[0],
		data, layers, l,
		matching = [];
	// If element is a canvas
	if (canvas && canvas.getContext) {
		
		data = _getCanvasData(canvas);
		// Retrieve layers array for this canvas
		layers = data.layers;
		matching = layers;
	}
	return matching;
};


/**
 * Get a single jCanvas layer object
 * @method getLayer
 * @param {String} layerId the id of the layer
 */
$.fn.getLayer = function getLayer(layerId) {
	var canvas = this[0],
		data = _getCanvasData(canvas),
		layer;
	
	// Get layer with the given name
	layer = data.layer.names[layerId];
		
	return layer;
};


/**
 * Get all layers in the given group
 * @method getLayerGroup
 * @param {String} groupId the id of the group
 */
$.fn.getLayerGroup = function getLayerGroup(groupId) {
	var canvas = this[0], group,data;
	
	// Find layer group with the given group name
	data = _getCanvasData(canvas);
	group = data.layer.groups[groupId];
		
	return group;
};

 
/**
 * Set properties of a layer
 * @method setLayer
 * @param {String} layerId the id of the layer
 * @param {object} props the new properties of the layer
 */
$.fn.setLayer = function setLayer(layerId, props) {
	var $canvases = this, $canvas, e,
		data, layer;
	
	$canvas = $($canvases.get(0));
	data = _getCanvasData($canvases.get(0));
		
	layer = $($canvases.get(0)).getLayer(layerId);
	if (layer) {
		// Merge properties with layer
		extendObject(layer, props);
	}
	
	return $canvases;
};

/**
 * Remove a jCanvas layer
 * @method removeLayer
 * @param {String} layerId the id of the layer
 */
$.fn.removeLayer = function removeLayer( layerId ) {
	var $canvases = this, $canvas, e, data,
		layers, layer;
		
		$canvas = $($canvases.get(0) );
		data = _getCanvasData( $canvases.get(0) );
		
		// Retrieve layers array and desired layer
		layers = $canvas.getLayers();
		layer = $canvas.getLayer( layerId );
		// Remove layer if found
		if ( layer ) {
			
			// Ensure layer index is accurate
			layer.index = inArray( layer, layers );
			layers.splice( layer.index, 1 );
			
			// Update layer name map
			_updateLayerName(data, layer, {
				name: NULL
			} );
			
			// Update layer group map
			_updateLayerGroups(data, layer, {
				groups: NULL
			} );
			
			// Delete name entry
			//delete data.layer.name[layer.name];
			
	}
	return $canvases;
};
 
/**
 * Remove all layers
 * @method removeLayers
 */
$.fn.removeLayers = function removeLayers( ) {
	var $canvases = this, $canvas,
		data, layers, layer, l;
	
		$canvas = $( $canvases.get(0) );
		data = _getCanvasData( $canvases.get(0) );
		layers = $canvas.getLayers( );
		// Remove all layers individually
		for ( l = 0; l < layers.length; l += 1 ) {
			layer = layers[ l ];
			$canvas.removeLayer( layer );
			// Ensure no layer is skipped over
		}
		// Update layer maps
		data.layer.names = {};
		data.layer.groups = {};
	return $canvases;
};


/**
 * Remove all layers in the group with the given ID
 * @method removeLayerGroup 
 * @param {String} groupId the id of the group
 */
$.fn.removeLayerGroup = function removeLayerGroup(groupId) {
	var $canvases = this, $canvas, data,
		layers, group, layer, l;
	
	if (groupId !== UNDEFINED) {

			$canvas = $($canvases.get(0));
			data = _getCanvasData($canvases.get(0));
			
			layers = $canvas.getLayers();
			group = $canvas.getLayerGroup(groupId);
			// Remove layer group using given group name
			if (group) {
				
				// Loop through layers in group
				for (l = 0; l < group.length; l += 1) {
					
					layer = group[l];
					// Ensure layer's index property is accurate
					layer.index = inArray(layer, layers);
					// Check if layer group matches name
					layers.splice(layer.index, 1);
					
					// Update layer name map
					_updateLayerName(data, layer, {
						name: NULL
					});
					
				}
				
				// Delete group entry
				delete data.layer.groups[group.name];
				
		}
	}
	return $canvases;
};

/**
 * Draw all layers (or, if given, only layers starting at an index)
 * @method drawLayers
 * @param {object} args the parameters defined to draw the layers
 */
$.fn.drawLayers = function drawLayers(args) {
	var $canvases = this, $canvas, ctx,
		// Internal parameters for redrawing the canvas
		params = extendObject({}, args),
		// Other variables
		layers, layer, l,
		data;
	// The layer index from which to start redrawing the canvas
	if (!params.index) {
		params.index = 0;
	}
		
	$canvas = $($canvases.get(0));
	ctx = _getContext($canvases.get(0));
	if (ctx) {
			
		data = _getCanvasData($canvases.get(0));
			
		// Clear canvas first unless otherwise directed
		if (params.clear !== FALSE) {
			$canvas.clearCanvas();
		}
			
		// Cache the layers array
		layers = data.layers;
			
		// Draw layers from first to last (bottom to top)
		for (l = params.index; l < layers.length; l += 1) {
			layer = layers[l];
				
			// Ensure layer index is up-to-date
			layer.index = l;
									
			// Prevent any one event from firing excessively
			if (params.resetFire) {
				layer._fired = FALSE;
			}
			
			// Draw layer
			layer._method.call($canvas, layer);
				
		}

	}
	return $canvases;
};


/**
 * Add a jCanvas layer (internal)
 * @method _addLayer
 * @param {object} canvas the canvas selected
 * @param {object} params
 * @param {object} args the parameters defined to draw the layers
 * @param {function} method the method to add
 * @private
 */
function _addLayer(canvas, params, args, method) {
	var $canvas, layers, layer = params,
		data;
	// Store arguments object for later use
	params._args = args;
	params.canvas = canvas;
	
	// Determine the layer's type using the available information
	if (method) {
		params._method = method;
	} else if (params.method) {
		params._method = $.fn[params.method];
	} else {
		params._method = function() {};
	}
		
	// If layer hasn't been added yet
	if (params.layer && !params._layer) {
		// Add layer to canvas
		
		$canvas = $(canvas);
				
		data = _getCanvasData(canvas);
		layers = data.layers;
		
		// Do not add duplicate layers of same name		
		if (layer.name === NULL || (isString(layer.name) && data.layer.names[layer.name] === UNDEFINED)) {
				
			// Ensure layers are unique across canvases by cloning them
			layer = params;
			// Indicate that this is a layer for future checks
			layer.layer = TRUE;
			layer._layer = TRUE;
			layer._data = data;
		
			// Update layer group maps
			_updateLayerName(data, layer);
			_updateLayerGroups(data, layer);
			
			// Add layer to end of array if no index is specified
			if (layer.index === NULL) {
				layer.index = layers.length;
			}
			
			// Add layer to layers array at specified index
			layers.splice(layer.index, 0, layer);
			
			// Store layer on parameters object
			params._args = layer;

		}
		
	}
	return layer;
}



/* Drawing API */

// Map drawing names with their respective method names
drawingMap = {
	'arc': 'drawArc',
	'bezier': 'drawBezier',
	'ellipse': 'drawEllipse',
	'function': 'draw',
	'line': 'drawLine',
	'path': 'drawPath',
	'polygon': 'drawPolygon',
	'slice': 'drawSlice',
	'quadratic': 'drawQuadratic',
	'rectangle': 'drawRect',
	'vector': 'drawVector'
};

 
/**
 * Draw on canvas using a function
 * @method draw
 * @param {object} args the parameters defined to draw the layers
 */
$.fn.draw = function draw( args ) {
	var $canvases = this, $canvas, ctx,
		params,
		layer;
			
		$canvas = $( $canvases.get(0) );
		ctx = _getContext( $canvases.get(0) );
		if ( ctx ) {
			
			params = new jCanvasObject(args);
			layer = _addLayer( $canvases.get(0), params, args, draw );
										
			if ( params.fn ) {
				// Call the given user-defined function
				params.fn.call( $canvases.get(0), ctx, params );
			}
	
	}
	return $canvases;
};


/**
 * Clear canvas
 * @method clearCanvas
 * @param {object} args the parameters defined to draw the layers
 */
$.fn.clearCanvas = function clearCanvas(args) {
	var $canvases = this,  ctx,
		params = new jCanvasObject(args),
		layer;
		
	ctx = _getContext($canvases.get(0));
	if (ctx) {
		// Reset current transformation temporarily to ensure that the entire canvas is cleared
		ctx.save();
		ctx.setTransform(1, 0, 0, 1, 0, 0);
		ctx.clearRect(0, 0, $canvases.get(0).width, $canvases.get(0).height);
		ctx.restore();
			
	}
	return $canvases;
};


/* Shape API */

/**
 * Draw rectangle
 * @method drawRect 
 * @param {object} args the parameters defined to draw the layers
 *
 */
$.fn.drawRect = function drawRect(args) {
	var $canvases = this,  ctx,
		params, layer,
		x1, y1, x2, y2, r;

	ctx = _getContext($canvases.get(0));
	if (ctx) {
			
		params = new jCanvasObject(args);
		layer = _addLayer($canvases.get(0), params, args, drawRect);

		_setGlobalProps( $canvases.get(0), ctx, params );
		
		ctx.beginPath();
		x1 = params.x - (params.width / 2);
		y1 = params.y - (params.height / 2);
		r = params.cornerRadius;
		// If corner radius is defined and is not zero
		if (r) {
			// Draw rectangle with rounded corners if cornerRadius is defined
					
			x2 = params.x + (params.width / 2);
			y2 = params.y + (params.height / 2);
			// Prevent over-rounded corners
			if ((x2 - x1) - (2 * r) < 0) {
				r = (x2 - x1) / 2;
			}
			if ((y2 - y1) - (2 * r) < 0) {
				r = (y2 - y1) / 2;
			}
			// Draw rectangle
			ctx.moveTo(x1 + r, y1);
			ctx.lineTo(x2 - r, y1);
			ctx.arc(x2 - r, y1 + r, r, 3 * PI / 2, PI * 2, FALSE);
			ctx.lineTo(x2, y2 - r);
			ctx.arc(x2 - r, y2 - r, r, 0, PI / 2, FALSE);
			ctx.lineTo(x1 + r, y2);
			ctx.arc(x1 + r, y2 - r, r, PI / 2, PI, FALSE);
			ctx.lineTo(x1, y1 + r);
			ctx.arc(x1 + r, y1 + r, r, PI, 3 * PI / 2, FALSE);
					
		}else {		
			// Otherwise draw rectangle with square corners
			ctx.rect(x1, y1, params.width, params.height);
					
		}

		// Close rectangle path
		ctx.fill();
		ctx.stroke();
		params.closed = TRUE;
		ctx.closePath();		
	}

	return $canvases;
};

 
/**
 * Draw arc or circle
 * @method drawArc
 * @param {object} args the parameters defined to draw the layers
 *
 */
$.fn.drawArc = function drawArc(args) {
	var $canvases = this,  ctx,
		params, layer,
		x1, y1, x2, y2,
		x3, y3, x4, y4,
		diff;
			
		ctx = _getContext($canvases.get(0));
		if (ctx) {
			
			params = new jCanvasObject(args);
			layer = _addLayer($canvases.get(0), params, args, drawArc);
			
			_setGlobalProps( $canvases.get(0), ctx, params );
				
			if (!params.inDegrees && params.end === 360) {
				// Convert default end angle to radians if necessary	
				params.end = PI * 2;
			}
				
			// Convert angles to radians
			params.start *= params._toRad;
			params.end *= params._toRad;
			// Consider 0deg due north of arc
			params.start -= (PI / 2);
			params.end -= (PI / 2);
				
			// Draw arc
			ctx.beginPath();
			ctx.arc(params.x, params.y, params.radius, params.start, params.end, params.ccw);
				
			diff = PI / 180 * 1;
			// Ensure arrows are pointed correctly for CCW arcs
			if (params.ccw) {
				diff *= -1;
			}
			// Calculate coordinates for start arrow
			x1 = _getX(params, params.start + diff);
			y1 = _getY(params, params.start + diff);
			x2 = _getX(params, params.start);
			y2 = _getY(params, params.start);
			// Calculate coordinates for end arrow
			x3 = _getX(params, params.end + diff);
			y3 = _getY(params, params.end + diff);
			x4 = _getX(params, params.end);
			y4 = _getY(params, params.end);
				
			// Optionally add arrows to path
			_addArrows(
				$canvases.get(0), ctx, params,
				x1, y1,
				x2, y2,
				x4, y4,
				x3, y3
			);

			// Optionally close path
			ctx.fill();
			ctx.stroke();
			params.closed = TRUE;
			ctx.closePath();
	}
	return $canvases;
};


/**
 * Draw ellipse
 * @method drawEllipse
 * @param {object} args the parameters defined to draw the layers
 *
 */
$.fn.drawEllipse = function drawEllipse(args) {
	var $canvases = this,  ctx,
		params, layer,
		controlW,
		controlH;
	
		ctx = _getContext( $canvases.get(0) );
		if ( ctx ) {

			params = new jCanvasObject( args );
			layer = _addLayer( $canvases.get(0), params, args, drawEllipse );
			
			_setGlobalProps( $canvases.get(0), ctx, params );

			// Calculate control width and height
			controlW = params.width * ( 4 / 3 );
			controlH = params.height;
				
			// Create ellipse using curves
			ctx.beginPath();
			ctx.moveTo( params.x, params.y - ( controlH / 2 ) );
			// Left side
			ctx.bezierCurveTo( params.x - ( controlW / 2 ), params.y - ( controlH / 2 ), params.x - ( controlW / 2 ), params.y + ( controlH / 2 ), params.x, params.y + ( controlH / 2 ) );
			// Right side
			ctx.bezierCurveTo( params.x + ( controlW / 2 ), params.y + ( controlH / 2 ), params.x + ( controlW / 2 ), params.y - ( controlH / 2 ), params.x, params.y - ( controlH / 2 ) );
				
			ctx.fill();
			ctx.stroke();
			params.closed = TRUE;
			ctx.closePath();

		}
	return $canvases;
};


/**
 * Draw a regular (equal-angled) polygon
 * @method drawPolygon
 * @param {object} args the parameters defined to draw the layers
 *
 */
$.fn.drawPolygon = function drawPolygon(args) {
	var $canvases = this,  ctx,
		params, layer,
		theta, dtheta, hdtheta,
		apothem,
		x, y, i;
	
		ctx = _getContext( $canvases.get(0) );
		if ( ctx ) {
			
			params = new jCanvasObject( args );
			layer = _addLayer( $canvases.get(0), params, args, drawPolygon );
			
			_setGlobalProps( $canvases.get(0), ctx, params );
			
			// Polygon's central angle
			dtheta = ( 2 * PI ) / params.sides;
			// Half of dtheta
			hdtheta = dtheta / 2;
			// Polygon's starting angle
			theta = hdtheta + ( PI / 2 );
			// Distance from polygon's center to the middle of its side
			apothem = params.radius * cos( hdtheta );
				
			// Calculate path and draw
			ctx.beginPath();
			for ( i = 0; i < params.sides; i += 1 ) {
					
				// Draw side of polygon
				x = params.x + ( params.radius * cos( theta ) );
				y = params.y + ( params.radius * sin( theta ) );
					
				// Plot point on polygon
				ctx.lineTo( x, y );
					
				// Project side if chosen
				if ( params.concavity ) {
					// Sides are projected from the polygon's apothem
					x = params.x + ( ( apothem + ( -apothem * params.concavity ) ) * cos( theta + hdtheta ) );
					y = params.y + ( ( apothem + ( -apothem * params.concavity ) ) * sin( theta + hdtheta ) );
					ctx.lineTo( x, y );
				}
					
				// Increment theta by delta theta
				theta += dtheta;
					
			}
			// Always close path
			ctx.fill();
			ctx.stroke();
			params.closed = TRUE;
			ctx.closePath();
				
			}
	return $canvases;
};


/**
 * Draw pie-shaped slice
 * @method drawSlice
 * @param {object} args the parameters defined to draw the layers
 *
 */
$.fn.drawSlice = function drawSlice(args) {
	var $canvases = this, $canvas,  ctx,
		params, layer,
		angle, dx, dy;
		
	$canvas = $($canvases.get(0));
	ctx = _getContext($canvases.get(0));
	if (ctx) {
			
		params = new jCanvasObject(args);
		layer = _addLayer($canvases.get(0), params, args, drawSlice);
		
		_setGlobalProps( $canvases.get(0), ctx, params );
				
		// Convert angles to radians										
		params.start *= params._toRad;
		params.end *= params._toRad;
		// Consider 0deg at north of arc
		params.start -= (PI / 2);
		params.end -= (PI / 2);
				
		// Find positive equivalents of angles
		params.start = _getCoterminal(params.start);
		params.end = _getCoterminal(params.end);
		// Ensure start angle is less than end angle
		if (params.end < params.start) {
				params.end += (2 * PI);
		}
				
		// Calculate angular position of slice
		angle = ((params.start + params.end) / 2);
				
		// Calculate ratios for slice's angle
		dx = (params.radius * params.spread * cos(angle));
		dy = (params.radius * params.spread * sin(angle));
				
		// Adjust position of slice
		params.x += dx;
		params.y += dy;
				
		// Draw slice
		ctx.beginPath();
		ctx.arc(params.x, params.y, params.radius, params.start, params.end, params.ccw);
		ctx.lineTo(params.x, params.y);

		// Always close path
		ctx.fill();
		ctx.stroke();
		params.closed = TRUE;
		ctx.closePath();
	
	}
	return $canvases;
};


/* Path API */

/**
 * Add arrow to path using the given properties
 * @method _addArrow
 * @param {object} canvas the canvas selected.
 * @param {object} ctx the 2D context of the canvas.
 * @param {object} params 
 * @param {object} path the line between source and target
 * @param {int} x1 the first abscissa 
 * @param {int} x2 the second abscissa 
 * @param {int} y1 the first ordinate 
 * @param {int} y2 the second ordinate  
 * @private
 *
 */
function _addArrow( canvas, ctx, params, path, x1, y1, x2, y2 ) {
	var leftX, leftY,
		rightX, rightY,
		offsetX, offsetY,
		angle;
	
	// If arrow radius is given and path is not closed
	if ( path.arrowRadius && !params.closed ) {
		
		// Calculate angle
		angle = atan2( ( y2 - y1 ), ( x2 - x1 ) );
		// Adjust angle correctly
		angle -= PI;
		// Calculate offset to place arrow at edge of path
		offsetX = ( params.strokeWidth * cos( angle ) );
		offsetY = ( params.strokeWidth * sin( angle ) );
		
		// Calculate coordinates for left half of arrow
		leftX = x2 + ( path.arrowRadius * cos( angle + ( path.arrowAngle / 2 ) ) );
		leftY = y2 + ( path.arrowRadius * sin( angle + ( path.arrowAngle / 2 ) ) );
		// Calculate coordinates for right half of arrow
		rightX = x2 + ( path.arrowRadius * cos( angle - ( path.arrowAngle / 2 ) ) );
		rightY = y2 + ( path.arrowRadius * sin( angle - ( path.arrowAngle / 2 ) ) );
		
		// Draw left half of arrow
		ctx.moveTo( leftX - offsetX, leftY - offsetY );
		ctx.lineTo( x2 - offsetX, y2 - offsetY );
		// Draw right half of arrow
		ctx.lineTo( rightX - offsetX, rightY - offsetY );
		
		// Visually connect arrow to path
		ctx.moveTo( x2 - offsetX, y2 - offsetY );
		ctx.lineTo( x2 + offsetX, y2 + offsetY );
		// Move back to end of path
		ctx.moveTo( x2, y2 );
		
	}
}

/**
 * Optionally add arrow to start of path
 * @method _addStartArrow
 * @param {object} canvas the canvas selected.
 * @param {object} ctx the 2D context of the canvas.
 * @param {object} params 
 * @param {object} path the line between source and target
 * @param {int} x1 the first abscissa 
 * @param {int} x2 the second abscissa 
 * @param {int} y1 the first ordinate 
 * @param {int} y2 the second ordinate  
 * @private
 *
 */
function _addStartArrow( canvas, ctx, params, path, x1, y1, x2, y2 ) {
	if ( !path._arrowAngleConverted ) {
		path.arrowAngle *= params._toRad;
		path._arrowAngleConverted = TRUE;
	}
	if ( path.startArrow ) {
		_addArrow( canvas, ctx, params, path, x1, y1, x2, y2 );
	}
}

/**
 * Optionally add arrow to end of path
 * @method _addEndArrow
 * @param {object} canvas the canvas selected.
 * @param {object} ctx the 2D context of the canvas.
 * @param {object} params 
 * @param {object} path the line between source and target
 * @param {int} x1 the first abscissa 
 * @param {int} x2 the second abscissa 
 * @param {int} y1 the first ordinate 
 * @param {int} y2 the second ordinate  
 * @private
 *
 */
function _addEndArrow( canvas, ctx, params, path, x1, y1, x2, y2 ) {
	if ( !path._arrowAngleConverted ) {
		path.arrowAngle *= params._toRad;
		path._arrowAngleConverted = TRUE;
	}
	if ( path.endArrow ) {
		_addArrow( canvas, ctx, params, path, x1, y1, x2, y2 );
	}
}

/**
 * Draw line ( internal )
 * @method _drawLine
 * @param {object} canvas the canvas selected.
 * @param {object} ctx the 2D context of the canvas.
 * @param {object} params 
 * @param {object} path the line between source and target
 * @private
 *
 */
function _drawLine( canvas, ctx, params, path ) {
	var l,
		lx, ly;
	
	l = 2;
	_addStartArrow(
		canvas, ctx,
		params, path,
		path.x2 + params.x,
		path.y2 + params.y,
		path.x1 + params.x,
		path.y1 + params.y
	);
	if ( path.x1 !== UNDEFINED && path.y1 !== UNDEFINED ) {
		ctx.moveTo( path.x1 + params.x, path.y1 + params.y );
	}
	while ( TRUE ) {
		// Calculate next coordinates
		lx = path[ 'x' + l ];
		ly = path[ 'y' + l ];
		// If coordinates are given
		if ( lx !== UNDEFINED && ly !== UNDEFINED ) {
			// Draw next line
			ctx.lineTo( lx + params.x, ly + params.y );
			l += 1;
		} else {
			// Otherwise, stop drawing
			break;
		}
	}
	l -= 1;
	// Optionally add arrows to path
	_addEndArrow(
		canvas, ctx,
		params,
		path,
		path[ 'x' + ( l - 1 ) ] + params.x,
		path[ 'y' + ( l - 1 ) ] + params.y,
		path[ 'x' + l ] + params.x,
		path[ 'y' + l ] + params.y
	);
}

/**
 * Draw line
 * @method drawLine
 * @param {object} args the parameters defined to draw the layers
 */
$.fn.drawLine = function drawLine( args ) {
	var $canvases = this,  ctx,
		params, layer;

	ctx = _getContext( $canvases.get(0) );
	if ( ctx ) {
			
		params = new jCanvasObject( args );
		layer = _addLayer( $canvases.get(0), params, args, drawLine );
	
		_setGlobalProps( $canvases.get(0), ctx, params );
		
		if(!params.replaceLine){
			// Draw each point
			ctx.beginPath();
			_drawLine( $canvases.get(0), ctx, params, params );

			// Always close path
			ctx.stroke();
			ctx.closePath();	
		}
		
			
	}
	return $canvases;
};
 
/**
 * Draw quadratic curve ( internal )
 * @method _drawQuadratic
 * @param {object} canvas the canvas selected.
 * @param {object} ctx the 2D context of the canvas.
 * @param {object} params 
 * @param {object} path the line between source and target
 * @private
 *
 */
function _drawQuadratic( canvas, ctx, params, path ) {
	var l,
		lx, ly,
		lcx, lcy;
	
	l = 2;
	
	_addStartArrow(
		canvas,
		ctx,
		params,
		path,
		path.cx1 + params.x,
		path.cy1 + params.y,
		path.x1 + params.x,
		path.y1 + params.y
	);
	
	if ( path.x1 !== UNDEFINED && path.y1 !== UNDEFINED ) {
		ctx.moveTo( path.x1 + params.x, path.y1 + params.y );
	}
	while ( TRUE ) {
		// Calculate next coordinates
		lx = path[ 'x' + l ];
		ly = path[ 'y' + l ];
		lcx = path[ 'cx' + ( l - 1 ) ];
		lcy = path[ 'cy' + ( l - 1 ) ];
		// If coordinates are given
		if ( lx !== UNDEFINED && ly !== UNDEFINED && lcx !== UNDEFINED && lcy !== UNDEFINED ) {
			// Draw next curve
			ctx.quadraticCurveTo( lcx + params.x, lcy + params.y, lx + params.x, ly + params.y );
			l += 1;
		} else {
			// Otherwise, stop drawing
			break;
		}
	}
	l -= 1;
	_addEndArrow(
		canvas,
		ctx,
		params,
		path,
		path[ 'cx' + ( l - 1 ) ] + params.x,
		path[ 'cy' + ( l - 1 ) ] + params.y,
		path[ 'x' + l ] + params.x,
		path[ 'y' + l ] + params.y
	);
}


/**
 * Draw quadratic curve
 * @method drawQuadratic
 * @param {object} args the parameters defined to draw the layers
 */
$.fn.drawQuadratic = function drawQuadratic( args ) {
	var $canvases = this,  ctx,
		params, layer;

	ctx = _getContext( $canvases.get(0) );
	if ( ctx ) {

		params = new jCanvasObject( args );
		layer = _addLayer( $canvases.get(0), params, args, drawQuadratic );
				
		_setGlobalProps( $canvases.get(0), ctx, params );
		// Draw each point
		ctx.beginPath();
		_drawQuadratic( $canvases.get(0), ctx, params, params );
				
		ctx.stroke();
		ctx.closePath();
			
	}
	return $canvases;
};

/**
 * Draw Bezier curve ( internal )
 * @method _drawBezier
 * @param {object} canvas the canvas selected.
 * @param {object} ctx the 2D context of the canvas.
 * @param {object} params 
 * @param {object} path the line between source and target
 * @private
 *
 */
function _drawBezier( canvas, ctx, params, path ) {
	var l, lc,
		lx, ly,
		lcx1, lcy1,
		lcx2, lcy2;
	
	l = 2;
	lc = 1;
	
	_addStartArrow(
		canvas,
		ctx,
		params,
		path,
		path.cx1 + params.x,
		path.cy1 + params.y,
		path.x1 + params.x,
		path.y1 + params.y
	);
	
	if ( path.x1 !== UNDEFINED && path.y1 !== UNDEFINED ) {
		ctx.moveTo( path.x1 + params.x, path.y1 + params.y );
	}
	
	while ( TRUE ) {
		// Calculate next coordinates
		lx = path[ 'x' + l ];
		ly = path[ 'y' + l ];
		lcx1 = path[ 'cx' + lc ];
		lcy1 = path[ 'cy' + lc ];
		lcx2 = path[ 'cx' + ( lc + 1 ) ];
		lcy2 = path[ 'cy' + ( lc + 1 ) ];
		// If next coordinates are given
		if ( lx !== UNDEFINED && ly !== UNDEFINED && lcx1 !== UNDEFINED && lcy1 !== UNDEFINED && lcx2 !== UNDEFINED && lcy2 !== UNDEFINED ) {
			// Draw next curve
			ctx.bezierCurveTo( lcx1 + params.x, lcy1 + params.y, lcx2 + params.x, lcy2 + params.y, lx + params.x, ly + params.y );
			l += 1;
			lc += 2;
		} else {
			// Otherwise, stop drawing
			break;
		}
	}
	l -= 1;
	lc -= 2;
	_addEndArrow(
		canvas,
		ctx,
		params,
		path,
		path[ 'cx' + ( lc + 1 ) ] + params.x,
		path[ 'cy' + ( lc + 1 ) ] + params.y,
		path[ 'x' + l ] + params.x,
		path[ 'y' + l ] + params.y
	);
}


/**
 *  Draw Bezier curve
 * @method drawBezier
 * @param {object} args the parameters defined to draw the layers
 */
$.fn.drawBezier = function drawBezier( args ) {
	var $canvases = this,  ctx,
		params, layer;

	ctx = _getContext( $canvases.get(0) );
	if ( ctx ) {

		params = new jCanvasObject( args );
		layer = _addLayer( $canvases.get(0), params, args, drawBezier );

		_setGlobalProps( $canvases.get(0), ctx, params );
				
		// Draw each point
		ctx.beginPath();
		_drawBezier( $canvases.get(0), ctx, params, params );
			
		// Optionally close path
		ctx.stroke();
		ctx.closePath();

	}
	return $canvases;
};


/**
 * Retrieves the x-coordinate for the given vector angle and length
 * @method drawBezier
 * @param {object} params
 * @param {int} angle the angle
 * @param {int} length the size of the vector
 * @private
 */
function _getVectorX( params, angle, length ) {
	angle *= params._toRad;
	angle -= ( PI / 2 );
	return ( length * cos( angle ) );
}
/**
 * Retrieves the y-coordinate for the given vector angle and length
 * @method drawBezier
 * @param {object} params
 * @param {int} angle the angle
 * @param {int} length the size of the vector
 * @private
 */
function _getVectorY( params, angle, length ) {
	angle *= params._toRad;
	angle -= ( PI / 2 );
	return ( length * sin( angle ) );
}
 
/**
 * Draw vector ( internal ) #2
 * @method _drawVector
 * @param {object} canvas the canvas selected.
 * @param {object} ctx the 2D context of the canvas.
 * @param {object} params 
 * @param {object} path the line between source and target
 * @private
 *
 */
function _drawVector( canvas, ctx, params, path ) {
	var l, angle, length,
		offsetX, offsetY,
		x, y,
		x2, y2,
		x3, y3,
		x4, y4;
	
	// Determine offset from dragging
	if ( params === path ) {
		offsetX = 0;
		offsetY = 0;
	} else {
		offsetX = params.x;
		offsetY = params.y;
	}
	
	l = 1;
	x = x2 = x3 = x4 = path.x + offsetX;
	y = y2 = y3 = y4 = path.y + offsetY;
	
	_addStartArrow(
		canvas, ctx,
		params, path,
		x + _getVectorX( params, path.a1, path.l1 ),
		y + _getVectorY( params, path.a1, path.l1 ),
		x,
		y
	);
	
	// The vector starts at the given ( x, y ) coordinates
	if ( path.x !== UNDEFINED && path.y !== UNDEFINED ) {
		ctx.moveTo( x, y );
	}
	while ( TRUE ) {
		
		angle = path[ 'a' + l ];
		length = path[ 'l' + l ];
		
		if ( angle !== UNDEFINED && length !== UNDEFINED ) {
			// Convert the angle to radians with 0 degrees starting at north
			// Keep track of last two coordinates
			x3 = x4;
			y3 = y4;
			// Compute ( x, y ) coordinates from angle and length
			x4 += _getVectorX( params, angle, length );
			y4 += _getVectorY( params, angle, length );
			// Store the second point
			if ( l === 1 ) {
				x2 = x4;
				y2 = y4;
			}
			ctx.lineTo( x4, y4 );
			l += 1;
		} else {
			// Otherwise, stop drawing
			break;
		}
		
	}
	_addEndArrow(
		canvas, ctx,
		params, path,
		x3, y3,
		x4, y4
	);
}


/**
 *  Draw vector
 * @method drawVector
 * @param {object} args the parameters defined to draw the layers
 */
$.fn.drawVector = function drawVector( args ) {
	var $canvases = this, ctx,
		params, layer;
	
	ctx = _getContext( $canvases.get(0) );
	if ( ctx ) {

		params = new jCanvasObject( args );
		layer = _addLayer( $canvases.get(0), params, args, drawVector );

		_setGlobalProps( $canvases.get(0), ctx, params );
		// Draw each point
		ctx.beginPath();
		_drawVector( $canvases.get(0), ctx, params, params );
		// Optionally close path
		ctx.stroke();
		ctx.closePath();

	};
	
	return $canvases;
};


/**
 * Draw a path consisting of one or more subpaths
 * @method drawPath
 * @param {object} args the parameters defined to draw the layers
 */
$.fn.drawPath = function drawPath( args ) {
	var $canvases = this, ctx,
		params, layer,
		l, lp;
		
	ctx = _getContext( $canvases.get(0) );
	if ( ctx ) {
			
		params = new jCanvasObject( args );
		layer = _addLayer( $canvases.get(0), params, args, drawPath );
			
		_setGlobalProps( $canvases.get(0), ctx, params );
		
		ctx.beginPath();
		l = 1;
		while ( TRUE ) {
			lp = params[ 'p' + l ];
			if ( lp !== UNDEFINED ) {
				lp = new jCanvasObject( lp );
				if ( lp.type === 'line' ) {
					_drawLine( $canvases.get(0), ctx, params, lp );
				} else if ( lp.type === 'quadratic' ) {
					_drawQuadratic( $canvases.get(0), ctx, params, lp );
				} else if ( lp.type === 'bezier' ) {
					_drawBezier( $canvases.get(0), ctx, params, lp );
				} else if ( lp.type === 'vector' ) {
					_drawVector( $canvases.get(0), ctx, params, lp );
				} else if ( lp.type === 'arc' ) {
					_drawArc( $canvases.get(0), ctx, params, lp );
				}
					l += 1;
				} else {
					break;
				}
			}
			
			// Optionally close path
			ctx.stroke();
			ctx.closePath();
			
	}
	return $canvases;
};
/**
 * Draw dashed line 
 * @method drawDashedLine
 * @param {object} canvas the canvas selected.
 * @param {object} ctx the 2D context of the canvas.
 * @param {object} params 
 *
 */
function drawDashedLine(canvas, ctx, args) {
  
    var $canvases = this,
		params, layer;


	if ( ctx ) {
			
		params = new jCanvasObject( args );
		layer = _addLayer( canvas, params, args, drawDashedLine );
		
		var x0 = params.x1;
		var y0 = params.y1;
		var x1 = params.x2;
		var y1 = params.y2;
		var pattern = params.strokeDash;
		
		// Draw each point
		ctx.beginPath();
	
    	var length = Math.sqrt(Math.pow(x1 - x0, 2) + Math.pow(y1 - y0, 2));
   	 	var vector = {
      		x: (x1 - x0) / length,
      		y: (y1 - y0) / length
   		 };
    	var dist = 0;
    	var i = 0;
  
    	pattern = pattern && pattern.length ? pattern : [4, 4];
    	while (dist < length) {
      		var dashLength = Math.min(pattern[i++ % pattern.length], length - dist);
      		var draw = (i % 2);
      		dist += dashLength;
  
      		draw && ctx.moveTo(x0, y0);
  
      		x0 += dashLength * vector.x;
      		y0 += dashLength * vector.y;
  
     	 	draw && ctx.lineTo(x0, y0);
    	}
    	
    	// Always close path
		ctx.stroke();
		ctx.closePath();
    	
    }
	return $canvases;

 };


// Enable canvas feature detection with $.support
$.support.canvas = ($('<canvas />')[0].getContext);

// Export jCanvas functions
jCanvas.defaults = defaults;
$.jCanvas = jCanvas;

}(jQuery, Math, parseFloat, true, false, null));